@@ -4,26 +4,16 @@
    find the most probable value in a HiSPARC spectrum
 
 """
-import datetime
-import urllib2
-import json
-import StringIO
 import warnings
 
-import numpy as np
-from scipy.stats import norm
 from scipy.optimize import curve_fit
-import pylab as plt
 
-from sapphire.api import Network
+from ..utils import gauss
+
+MPV_FIT_WIDTH_FACTOR = 0.4
 
 
-HIST_URL = 'http://data.hisparc.nl/show/source/pulseintegral/%d/%d/%d/%d/'
-
-MPV_FIT_WIDTH_FACTOR = .4
-
-
-class FindMostProbableValueInSpectrum(object):
+class FindMostProbableValueInSpectrum:
     """Find the most probable value (MPV) in a HiSPARC spectrum.
 
     This is a fast algorithm to find the MPV value in a HiSPARC spectrum.
@@ -50,10 +40,10 @@ class FindMostProbableValueInSpectrum(object):
     """
 
     def __init__(self, n, bins):
-        """Initialize the class instance.
+        """Initialize the class instance
 
-        :param n, bins: histogram counts and bins, as obtained using
-            :func:`numpy.histogram`.
+        :param n,bins: histogram counts and bins, as obtained using
+            ``numpy.histogram``.
 
         """
         self.n, self.bins = n, bins
@@ -64,7 +54,7 @@ class FindMostProbableValueInSpectrum(object):
         First perform a first guess, then use that value to fit the MIP
         peak.
 
-        :return mpv: best guess of the most probable value
+        :return mpv: best guess of the most probable value.
         :return boolean is_fitted: indicates if the fit was successful.
 
         """
@@ -92,7 +82,7 @@ class FindMostProbableValueInSpectrum(object):
            * Find the maximum *to the right* of this value.  We assume
              this to be the approximate location of the MIP peak.
 
-        :returns mpv: first guess of the most probable value
+        :return mpv: first guess of the most probable value.
 
         """
         n, bins = self.n, self.bins
@@ -123,12 +113,12 @@ class FindMostProbableValueInSpectrum(object):
         be adjusted by the width_factor parameter.
 
         :param first_guess: approximate location of the most probable
-            value
+            value.
         :param width_factor: float in the range [0., 1.] to indicate the
             width of the fit domain.  The domain is given by
-            [(1. - width_factor) * first_guess, (1. + width_factor) *
-            first_guess]
-        :returns mpv: mpv value obtained from the fit
+            ``[(1. - width_factor) * first_guess, (1. + width_factor) *
+            first_guess]``.
+        :return mpv: mpv value obtained from the fit.
 
         """
         n, bins = self.n, self.bins
@@ -149,9 +139,8 @@ class FindMostProbableValueInSpectrum(object):
             raise RuntimeError("Number of data points not sufficient")
 
         # fit to a normal distribution
-        f = lambda x, N, a, b: N * norm.pdf(x, loc=a, scale=b)
-        popt, pcov = curve_fit(f, x, y, p0=(y.max(), first_guess,
-                                            first_guess))
+        popt, pcov = curve_fit(gauss, x, y,
+                               p0=(y.max(), first_guess, first_guess))
         mpv = popt[1]
 
         # sanity check: if MPV is outside domain, the MIP peak was not
@@ -160,68 +149,3 @@ class FindMostProbableValueInSpectrum(object):
             raise RuntimeError("Fitted MPV value outside range")
 
         return mpv
-
-
-def main():
-    """Demo the MPV finder with actual data."""
-
-    today = datetime.date.today()
-    yesterday = today - datetime.timedelta(days=1)
-    station_ids = get_station_ids_with_data(yesterday)
-
-    for station in station_ids:
-        if station == 10:
-            continue
-        print station
-        n, bins = get_histogram_for_station_on_date(station, yesterday)
-        find_mpv = FindMostProbableValueInSpectrum(n, bins)
-        mpv, is_fitted = find_mpv.find_mpv()
-
-        plt.figure()
-        plt.plot((bins[:-1] + bins[1:]) / 2., n)
-        if is_fitted:
-            plt.axvline(mpv, c='g')
-        else:
-            plt.axvline(mpv, c='r')
-        plt.title(station)
-        plt.yscale('log')
-
-
-def get_station_ids_with_data(date):
-    """Return the station ids having data on this day."""
-
-    station_list = Network.stations_with_data(date.year, date.month, date.day)
-    station_ids = [int(station['number']) for station in station_list]
-
-    return station_ids
-
-
-def get_histogram_for_station_on_date(station_id, date):
-    """Return a histogram of the spectrum of a station on a date.
-
-    :return n, bins: histogram counts and bins, as obtained using
-       :func:`numpy.histogram`.
-
-    """
-    url = HIST_URL % (station_id, date.year, date.month, date.day)
-
-    reply = urllib2.urlopen(url)
-    reply = reply.read()
-
-    file_like = StringIO.StringIO(reply)
-    data = np.genfromtxt(file_like)
-
-    bins = data[:, 0]
-    bins = list(bins)
-    bins.append(bins[-1] + (bins[-1] - bins[-2]))
-    bins = np.array(bins)
-
-    n = data[:, 1]
-
-    return n, bins
-
-
-if __name__ == '__main__':
-    warnings.simplefilter('always')
-    main()
-    plt.show()
