@@ -10,8 +10,7 @@
         import tables
 
         from sapphire.publicdb import download_data
-        from sapphire.analysis import process_events
-
+        from sapphire import ProcessEvents
 
         STATIONS = [501, 503, 506]
         START = datetime.datetime(2013, 1, 1)
@@ -21,33 +20,38 @@
         if __name__ == '__main__':
             station_groups = ['/s%d' % u for u in STATIONS]
 
-            data = tables.open_file('data.h5', 'w')
-            for station, group in zip(STATIONS, station_groups):
-                download_data(data, group, station, START, END, get_blobs=True)
-                proc = process_events.ProcessEvents(data, group)
-                proc.process_and_store_results()
-            data.close()
+            with tables.open_file('data.h5', 'w') as data:
+                for station, group in zip(STATIONS, station_groups):
+                    download_data(data, group, station, START, END, True)
+                    proc = ProcessEvents(data, group)
+                    proc.process_and_store_results()
 
 """
-import zlib
-from itertools import izip
 import operator
+import os
+import warnings
+import zlib
 
-import tables
 import numpy as np
-from scipy.optimize import curve_fit
+import tables
 
-from ..utils import pbar, gauss, ERR
+from ..api import Station
+from ..utils import ERR, pbar
 from .find_mpv import FindMostProbableValueInSpectrum
+from .process_traces import ADC_HIGH_THRESHOLD, ADC_LOW_THRESHOLD, ADC_TIME_PER_SAMPLE
+
+ADC_THRESHOLD = 20  #: Threshold for arrival times, relative to the baseline
+ADC_LIMIT = 2 ** 12
+
+#: Default trigger for 2-detector station
+#: 2 low and no high, no external
+TRIGGER_2 = (2, 0, False, 0)
+#: Default trigger for 4-detector station
+#: 3 low or 2 high, no external
+TRIGGER_4 = (3, 2, True, 0)
 
 
-ADC_THRESHOLD = 20  # This one is relative to the baseline
-ADC_LOW_THRESHOLD = 253
-ADC_HIGH_THRESHOLD = 323
-ADC_TIME_PER_SAMPLE = 2.5  # in ns
-
-
-class ProcessEvents(object):
+class ProcessEvents:
 
     """Process HiSPARC events to obtain several observables.
 
@@ -89,13 +93,15 @@ class ProcessEvents(object):
             cases, this is simply the group containing the events table.
         :param source: the name of the events table.  Default: None,
             meaning the default name 'events'.
-        :param progress: show progressbar.
+        :param progress: if True show a progressbar while copying and
+                         processing events.
 
         """
         self.data = data
         self.group = data.get_node(group)
         self.source = self._get_source(source)
         self.progress = progress
+        self.limit = None
 
     def process_and_store_results(self, destination=None, overwrite=False,
                                   limit=None):
@@ -123,7 +129,7 @@ class ProcessEvents(object):
         """Return the traces from an event.
 
         :param event: a row from the events table.
-        :returns: the traces: an array of pulseheight values.
+        :return: the traces: an array of pulseheight values.
 
         """
         traces = [list(self._get_trace(idx)) for idx in event['traces']
@@ -137,7 +143,7 @@ class ProcessEvents(object):
         """Return the traces from event #idx.
 
         :param idx: the index number of the event.
-        :returns: the traces: an array of pulseheight values.
+        :return: the traces: an array of pulseheight values.
 
         """
         event = self.source[idx]
@@ -149,7 +155,7 @@ class ProcessEvents(object):
         :param source: the *name* of the table.  If None, this method will
             try to find the original events table, even if the events were
             previously processed.
-        :returns: table object
+        :return: table object
 
         """
         if source is None:
@@ -186,7 +192,6 @@ class ProcessEvents(object):
 
         """
         events = self.source
-        events_tablename = self.source.name
 
         enumerated_timestamps = list(enumerate(events.col('ext_timestamp')))
         enumerated_timestamps.sort(key=operator.itemgetter(1))
@@ -238,7 +243,7 @@ class ProcessEvents(object):
         :param events: the events table to normalize.
 
         """
-        row_ids = range(len(events))
+        row_ids = list(range(len(events)))
         events.modify_column(column=row_ids, colname='event_id')
 
     def _create_results_table(self):
@@ -261,7 +266,7 @@ class ProcessEvents(object):
                                        self.processed_events_description,
                                        expectedrows=length)
 
-        for x in xrange(length):
+        for _ in range(length):
             table.row.append()
         table.flush()
 
@@ -272,8 +277,8 @@ class ProcessEvents(object):
         source = self.source
 
         for col in pbar(source.colnames, show=self.progress):
-            getattr(table.cols, col)[:self.limit] = getattr(source.cols,
-                                                            col)[:self.limit]
+            table.modify_column(stop=self.limit, colname=col,
+                                column=getattr(source.cols, col)[:self.limit])
         table.flush()
 
     def _store_results_from_traces(self):
@@ -284,16 +289,13 @@ class ProcessEvents(object):
         # Assign values to full table, column-wise.
         for idx in range(4):
             col = 't%d' % (idx + 1)
-            getattr(table.cols, col)[:] = timings[:, idx]
+            table.modify_column(column=timings[:, idx], colname=col)
         table.flush()
 
-    def process_traces(self, limit=None):
+    def process_traces(self):
         """Process traces to yield pulse timing information."""
 
-        if limit:
-            self.limit = limit
-
-        if self.limit:
+        if self.limit is not None:
             events = self.source.iterrows(stop=self.limit)
         else:
             events = self.source
@@ -326,8 +328,8 @@ class ProcessEvents(object):
         This method loops over the traces.
 
         :param event: row from the events table.
-        :returns: arrival times in the detectors relative to trace start
-                  in ns.
+        :return: arrival times in the detectors relative to trace start
+                 in ns.
 
         """
         timings = []
@@ -354,15 +356,16 @@ class ProcessEvents(object):
         Decompress a trace from the blobs array.
 
         :param idx: index into the blobs array
-        :returns: iterator over the pulseheight values
+        :return: iterator over the pulseheight values
 
         """
         blobs = self._get_blobs()
 
         try:
-            trace = zlib.decompress(blobs[idx]).split(',')
+            trace = zlib.decompress(blobs[idx]).decode('utf-8').split(',')
         except zlib.error:
-            trace = zlib.decompress(blobs[idx][1:-1]).split(',')
+            trace = (zlib.decompress(blobs[idx][1:-1])
+                     .decode('utf-8').split(','))
         if trace[-1] == '':
             del trace[-1]
         trace = (int(x) for x in trace)
@@ -378,7 +381,7 @@ class ProcessEvents(object):
 
         :param trace: array containing pulseheight values.
         :param baseline: baseline of the trace.
-        :returns: index in trace for arrival time of first particle.
+        :return: index in trace for arrival time of first particle.
 
         """
         threshold = baseline + ADC_THRESHOLD
@@ -392,13 +395,18 @@ class ProcessEvents(object):
 
         If no element matches the condition -999 will be returned.
 
+        :param trace: iterable trace.
+        :param threshold: value the trace has to be greater or equal to.
+        :return: index in trace where a value is greater or equal to
+                 threshold.
+
         """
         return next((i for i, x in enumerate(trace) if x >= threshold), -999)
 
     def _store_number_of_particles(self):
         """Store number of particles in the detectors.
 
-        Process all pulseheights from the events and estimate the number
+        Process all pulseintegrals from the events and estimate the number
         of particles in each detector.
 
         """
@@ -407,10 +415,16 @@ class ProcessEvents(object):
         n_particles = self._process_pulseintegrals()
         for idx in range(4):
             col = 'n%d' % (idx + 1)
-            getattr(table.cols, col)[:] = n_particles[:, idx]
+            table.modify_column(column=n_particles[:, idx], colname=col)
         table.flush()
 
     def _process_pulseintegrals(self):
+        """Find MPVs using pulseintegrals to estimate number of particles
+
+        :return: array with estimated number of particles per detector per
+                 event.
+
+        """
         n_particles = []
 
         integrals = self.source.col('integrals')
@@ -451,27 +465,14 @@ class ProcessEvents(object):
             self.data.remove_node(self.group, self.destination)
         self._tmp_events.rename(self.destination)
 
-    def determine_detector_timing_offsets(self, timings_table='events'):
-        """Determine the offsets between the station detectors."""
-
-        table = self.data.get_node(self.group, timings_table)
-        t2 = table.col('t2')
-
-        bins = np.arange(-100 + 1.25, 100, 2.5)
-
-        print "Determining offsets based on # events:",
-        offsets = []
-        for timings in 't1', 't3', 't4':
-            timings = table.col(timings)
-            dt = (timings - t2).compress((t2 >= 0) & (timings >= 0))
-            print len(dt),
-            y, bins = np.histogram(dt, bins=bins)
-            x = (bins[:-1] + bins[1:]) / 2
-            popt, pcov = curve_fit(gauss, x, y, p0=(len(dt), 0., 10.))
-            offsets.append(popt[1])
-        print
-
-        return [offsets[0]] + [0.] + offsets[1:]
+    def __repr__(self):
+        if not self.data.isopen:
+            return "<finished %s>" % self.__class__.__name__
+        else:
+            return ("%s(%r, %r, source=%r, progress=%r)" %
+                    (self.__class__.__name__, self.data.filename,
+                     self.group._v_pathname, self.source._v_pathname,
+                     self.progress))
 
 
 class ProcessIndexedEvents(ProcessEvents):
@@ -493,19 +494,19 @@ class ProcessIndexedEvents(ProcessEvents):
         :param indexes: a list of indexes into the events table.
         :param source: the name of the events table.  Default: None,
             meaning the default name 'events'.
+        :param progress: if True show a progressbar while copying and
+                         processing events.
 
         """
-        super(ProcessIndexedEvents, self).__init__(data, group, source)
+        super().__init__(data, group, source, progress)
         self.indexes = indexes
-        self.progress = progress
 
     def _store_results_from_traces(self):
         table = self._tmp_events
 
         timings = self.process_traces()
 
-        for event, (t1, t2, t3, t4) in izip(table.itersequence(self.indexes),
-                                            timings):
+        for event, (t1, t2, t3, t4) in zip(table.itersequence(self.indexes), timings):
             event['t1'] = t1
             event['t2'] = t2
             event['t3'] = t3
@@ -547,7 +548,7 @@ class ProcessEventsWithLINT(ProcessEvents):
 
         :param trace: array containing pulseheight values.
         :param baseline: baseline of the trace
-        :returns: arrival time
+        :return: arrival time
 
         """
         threshold = baseline + ADC_THRESHOLD
@@ -566,13 +567,12 @@ class ProcessEventsWithLINT(ProcessEvents):
         return value
 
 
-class ProcessIndexedEventsWithLINT(ProcessIndexedEvents,
-                                   ProcessEventsWithLINT):
+class ProcessIndexedEventsWithLINT(ProcessIndexedEvents, ProcessEventsWithLINT):
 
     """Process a subset of events using LInear INTerpolation.
 
     This is a subclass of :class:`ProcessIndexedEvents` and
-    :class:`ProcessEventsWithLint`.
+    :class:`ProcessEventsWithLINT`.
 
     """
 
@@ -597,8 +597,7 @@ class ProcessEventsWithoutTraces(ProcessEvents):
         pass
 
 
-class ProcessIndexedEventsWithoutTraces(ProcessEventsWithoutTraces,
-                                        ProcessIndexedEvents):
+class ProcessIndexedEventsWithoutTraces(ProcessEventsWithoutTraces, ProcessIndexedEvents):
 
     """Process a subset of events without traces
 
@@ -626,6 +625,33 @@ class ProcessEventsWithTriggerOffset(ProcessEvents):
 
     """
 
+    def __init__(self, data, group, source=None, progress=True, station=None):
+        """Initialize the class.
+
+        :param data: the PyTables datafile
+        :param group: the group containing the station data.  In normal
+            cases, this is simply the group containing the events table.
+        :param source: the name of the events table.  Default: None,
+            meaning the default name 'events'.
+        :param progress: if True show a progressbar while copying and
+                         processing events.
+        :param station: station number of station to which the data belongs.
+
+        """
+        super().__init__(data, group, source, progress)
+        if station is None:
+            self.station = None
+            self.thresholds = [(ADC_LOW_THRESHOLD, ADC_HIGH_THRESHOLD)] * 4
+            n = sum(1 for idx in self.source[0]['traces'] if idx != -1)
+            if n == 2:
+                self.trigger = TRIGGER_2
+            elif n == 4:
+                self.trigger = TRIGGER_4
+            else:
+                raise Exception('No trigger settings available')
+        else:
+            self.station = Station(station)
+
     def _store_results_from_traces(self):
         table = self._tmp_events
 
@@ -634,8 +660,8 @@ class ProcessEventsWithTriggerOffset(ProcessEvents):
         # Assign values to full table, column-wise.
         for idx in range(4):
             col = 't%d' % (idx + 1)
-            getattr(table.cols, col)[:] = timings[:, idx]
-        getattr(table.cols, 't_trigger')[:] = timings[:, 4]
+            table.modify_column(column=timings[:, idx], colname=col)
+        table.modify_column(column=timings[:, 4], colname='t_trigger')
         table.flush()
 
     def _reconstruct_time_from_traces(self, event):
@@ -644,49 +670,67 @@ class ProcessEventsWithTriggerOffset(ProcessEvents):
         This method loops over the traces.
 
         :param event: row from the events table.
-        :returns: arrival times in the detectors and trigger time
-                  relative to start of trace in ns
+        :return: arrival times in the detectors and trigger time
+                 relative to start of trace in ns
 
         """
+        if self.station is not None:
+            timestamp = event['timestamp']
+            try:
+                self.thresholds, self.trigger = self.station.trigger(timestamp)
+            except Exception:
+                warnings.warn('Unknown trigger settings, not reconstructing trigger offset.')
+                # Do not reconstruct t_trigger by pretending external trigger.
+                self.trigger = [0, 0, 0, 1]
+
+        n_low, n_high, and_or, external = self.trigger
+
+        if external:
+            # Do not reconstruct thresholds if external trigger is involved
+            self.thresholds = [(ADC_LIMIT, ADC_LIMIT)] * 4
+
         timings = []
         low_idx = []
         high_idx = []
-        n_detectors = sum(1 for idx in event['traces'] if idx != -1)
-        for baseline, pulseheight, trace_idx in zip(event['baseline'],
-                                                    event['pulseheights'],
-                                                    event['traces']):
+        for baseline, pulseheight, trace_idx, trig_thresholds in zip(
+                event['baseline'], event['pulseheights'], event['traces'],
+                self.thresholds):
             if pulseheight < 0:
-                # retain -1, -999 status flags in timing
+                # Retain -1 and -999 status flags in timing
                 timings.append(pulseheight)
+                low_idx.append(-999)
+                high_idx.append(-999)
+                continue
+            if pulseheight < ADC_THRESHOLD or baseline > trig_thresholds[0]:
+                # No significant pulse or bad baseline
+                timings.append(-999)
+                low_idx.append(-999)
+                high_idx.append(-999)
                 continue
 
             max_signal = baseline + pulseheight
             adc_threshold = baseline + ADC_THRESHOLD
 
-            if max_signal < adc_threshold and max_signal < ADC_LOW_THRESHOLD:
-                # No significant pulse
-                timings.append(-999)
-                continue
-            elif baseline > ADC_LOW_THRESHOLD:
-                # Bad baseline
-                timings.append(-999)
-                continue
+            thresholds = [adc_threshold]
+            # Only include if needed for trigger and large enough signal
+            if n_low and max_signal >= trig_thresholds[0]:
+                thresholds.append(trig_thresholds[0])
+            else:
+                thresholds.append(ADC_LIMIT)
+            if n_high and max_signal >= trig_thresholds[1]:
+                thresholds.append(trig_thresholds[1])
+            else:
+                thresholds.append(ADC_LIMIT)
 
             trace = self._get_trace(trace_idx)
 
-            if n_detectors == 2:
-                thresholds = (adc_threshold, ADC_LOW_THRESHOLD)
-            else:
-                thresholds = (adc_threshold, ADC_LOW_THRESHOLD,
-                              ADC_HIGH_THRESHOLD)
-
             t, l, h = self._first_above_thresholds(trace, thresholds,
                                                    max_signal)
             timings.append(t)
             low_idx.append(l)
             high_idx.append(h)
 
-        t_trigger = self._reconstruct_trigger(low_idx, high_idx, n_detectors)
+        t_trigger = self._reconstruct_trigger(low_idx, high_idx)
         timings.append(t_trigger)
 
         timings = [time * ADC_TIME_PER_SAMPLE if time not in ERR else time
@@ -699,19 +743,20 @@ class ProcessEventsWithTriggerOffset(ProcessEvents):
 
         First the thresholds are sorted to make sure they are looked for
         in the correct order, because the trace is a generator you can
-        not go back.
+        not go back. The results will match the original order.
 
         :param trace: generator over the trace.
-        :param thresholds: list of thresholds.
+        :param thresholds: list of three thresholds.
         :param max_signal: expected max value in trace, based on
-                           baseline and pulseheight
-        :returns: list with three indexes into the trace for the three
-                  thresholds
+                           baseline and pulseheight.
+        :return: list with three indexes into the trace for the three
+                 thresholds.
 
         """
         results = [-999, -999, -999]
-        ordered_thresholds = sorted([(x, i) for i, x in enumerate(thresholds)])
+        ordered_thresholds = sorted((x, i) for i, x in enumerate(thresholds))
         last_value = None
+        t = 0
 
         for threshold, i in ordered_thresholds:
             if max_signal < threshold:
@@ -721,10 +766,7 @@ class ProcessEventsWithTriggerOffset(ProcessEvents):
             else:
                 if last_value is not None:
                     t += 1
-                else:
-                    t = 0
-                t, last_value = cls._first_value_above_threshold(trace,
-                                                                 threshold, t)
+                t, last_value = cls._first_value_above_threshold(trace, threshold, t)
                 results[i] = t
         return results
 
@@ -742,34 +784,61 @@ class ProcessEventsWithTriggerOffset(ProcessEvents):
         return next(((i, x) for i, x in enumerate(trace, t) if x >= threshold),
                     (-999, 0))
 
-    def _reconstruct_trigger(self, low_idx, high_idx, n_detectors=None):
+    def _reconstruct_trigger(self, low_idx, high_idx):
         """Reconstruct the moment of trigger from the threshold info
 
-        :param low_idx, high_idx: list of trace indexes when a detector
-                                   crossed a given threshold.
-        :param n_detectors: number of detectors (2 or 4).
-        :returns: index in trace where the trigger happened.
+        :param low_idx,high_idx: list of trace indexes when a detector
+                                 crossed a given threshold.
+        :return: index in trace where the trigger happened.
 
         """
-        low_idx = [idx for idx in low_idx if not idx == -999]
-        high_idx = [idx for idx in high_idx if not idx == -999]
-        low_idx.sort()
-        high_idx.sort()
+        n_low, n_high, and_or, external = self.trigger
 
-        if n_detectors == 2 and len(low_idx) > 1:
-            # Two low
-            return low_idx[1]
-        elif n_detectors == 4 and (len(low_idx) >= 3 or len(high_idx) >= 2):
-            # Two high or three low
-            if len(low_idx) < 3 and len(high_idx) >= 2:
-                return high_idx[1]
-            elif len(low_idx) >= 3 and len(high_idx) < 2:
-                return low_idx[2]
-            elif len(low_idx) >= 3 and len(high_idx) >= 2:
-                return min([low_idx[2], high_idx[1]])
-        else:
+        # External trigger not supported
+        if external:
             return -999
 
+        low_idx = sorted(idx for idx in low_idx if not idx == -999)
+        high_idx = sorted(idx for idx in high_idx if not idx == -999)
+
+        if and_or:
+            # low or high, which ever is first
+            if n_low and n_high and len(low_idx) >= n_low and len(high_idx) >= n_high:
+                return min(low_idx[n_low - 1], high_idx[n_high - 1])
+            elif n_high and len(high_idx) >= n_high:
+                return high_idx[n_high - 1]
+            elif n_low and len(low_idx) >= n_low:
+                return low_idx[n_low - 1]
+        else:
+            if n_low and n_high:
+                # low and high
+                if len(low_idx) >= n_low + n_high and len(high_idx) >= n_high:
+                    return max(low_idx[n_low + n_high - 1], high_idx[n_high - 1])
+            elif n_high:
+                # 0 low and high
+                if len(high_idx) >= n_high:
+                    return high_idx[n_high - 1]
+            elif n_low:
+                # low and 0 high
+                if len(low_idx) >= n_low:
+                    return low_idx[n_low - 1]
+
+        return -999
+
+    def __repr__(self):
+        if not self.data.isopen:
+            return "<finished %s>" % self.__class__.__name__
+        elif self.station is None:
+            return ("%s(%r, %r, source=%r, progress=%r, Station=%r)" %
+                    (self.__class__.__name__, self.data.filename,
+                     self.group._v_pathname, self.source._v_pathname,
+                     self.progress, None))
+        else:
+            return ("%s(%r, %r, source=%r, progress=%r, station=%d)" %
+                    (self.__class__.__name__, self.data.filename,
+                     self.group._v_pathname, self.source._v_pathname,
+                     self.progress, self.station.number))
+
 
 class ProcessEventsFromSource(ProcessEvents):
 
@@ -782,24 +851,37 @@ class ProcessEventsFromSource(ProcessEvents):
 
     """
 
-    def __init__(self, source_file, dest_file, source_group, dest_group):
+    def __init__(self, source_file, dest_file, source_group, dest_group,
+                 progress=False):
         """Initialize the class.
 
-        :param source_file: the PyTables source file
-        :param dest_file: the PyTables dest file
-        :param group_path: the pathname of the source (and destination)
-            group
+        :param source_file,dest_file: PyTables source and destination files.
+        :param source_group,dest_group: the pathname of the source and
+                                        destination group.
+        :param progress: if True show a progressbar while copying and
+                         processing events.
 
         """
         self.source_file = source_file
         self.dest_file = dest_file
 
         self.source_group = self.source_file.get_node(source_group)
-        self.dest_group = self.dest_file.get_node(dest_group)
+        self.dest_group = self._get_or_create_group(dest_file, dest_group)
 
         self.source = self._get_source()
 
-        self.progress = False
+        self.progress = progress
+        self.limit = None
+
+    def _get_or_create_group(self, file, group):
+        """Get or create a group in the datafile"""
+
+        try:
+            group = file.get_node(group)
+        except tables.NoSuchNodeError:
+            parent, newgroup = os.path.split(group)
+            group = file.create_group(parent, newgroup, createparents=True)
+        return group
 
     def _get_source(self):
         """Return the table containing the events.
@@ -844,7 +926,7 @@ class ProcessEventsFromSource(ProcessEvents):
                                             self.processed_events_description,
                                             expectedrows=length)
 
-        for x in xrange(length):
+        for _ in range(length):
             table.row.append()
         table.flush()
 
@@ -859,6 +941,15 @@ class ProcessEventsFromSource(ProcessEvents):
 
         return self.source_group.blobs
 
+    def __repr__(self):
+        if not self.source_file.isopen or not self.dest_file.isopen:
+            return f"<finished {self.__class__.__name__}>"
+        else:
+            return ("%s(%r, %r, %r, %r, progress=%r)" %
+                    (self.__class__.__name__, self.source_file.filename,
+                     self.dest_file.filename, self.source_group._v_pathname,
+                     self.dest_group._v_pathname, self.progress))
+
 
 class ProcessEventsFromSourceWithTriggerOffset(ProcessEventsFromSource,
                                                ProcessEventsWithTriggerOffset):
@@ -868,8 +959,277 @@ class ProcessEventsFromSourceWithTriggerOffset(ProcessEventsFromSource,
     This is a subclass of :class:`ProcessEventsFromSource` and
     :class:`ProcessEventsWithTriggerOffset`.  Processing events and
     finding the trigger time in the traces. And storing the results in a
-    different file that the source.
+    different file than the source.
 
     """
 
-    pass
+    def __init__(self, source_file, dest_file, source_group, dest_group,
+                 station=None, progress=False):
+        """Initialize the class.
+
+        :param source_file,dest_file: PyTables source and destination files.
+        :param source_group,dest_group: the pathname of the source and
+                                        destination group.
+        :param station: station number of station to which the data belongs.
+        :param progress: if True show a progressbar while copying and
+                         processing events.
+
+        """
+        self.source_file = source_file
+        self.dest_file = dest_file
+
+        self.source_group = self.source_file.get_node(source_group)
+        self.dest_group = self._get_or_create_group(dest_file, dest_group)
+
+        self.source = self._get_source()
+
+        self.progress = progress
+        self.limit = None
+
+        if station is None:
+            self.station = None
+            self.thresholds = [(ADC_LOW_THRESHOLD, ADC_HIGH_THRESHOLD)] * 4
+            n = sum(1 for idx in self.source[0]['traces'] if idx != -1)
+            if n == 2:
+                self.trigger = TRIGGER_2
+            elif n == 4:
+                self.trigger = TRIGGER_4
+            else:
+                raise Exception('No trigger settings available')
+        else:
+            self.station = Station(station)
+
+    def __repr__(self):
+        if not self.source_file.isopen or not self.dest_file.isopen:
+            return "<finished %s>" % self.__class__.__name__
+        elif self.station is None:
+            return ("%s(%r, %r, %r, %r, progress=%r)" %
+                    (self.__class__.__name__, self.source_file.filename,
+                     self.dest_file.filename, self.source_group._v_pathname,
+                     self.dest_group._v_pathname, self.progress))
+        else:
+            return ("%s(%r, %r, %r, %r, station=%d, progress=%r)" %
+                    (self.__class__.__name__, self.source_file.filename,
+                     self.dest_file.filename, self.source_group._v_pathname,
+                     self.dest_group._v_pathname, self.station.number,
+                     self.progress))
+
+
+class ProcessDataTable(ProcessEvents):
+
+    """Process HiSPARC abstract data table to clean the data.
+
+    Abstract data is a PyTables table containing a timestamp for each row.
+    Weather and singles data are examples of such tables. This class can be
+    used to process a set of abstract HiSPARC data, to remove duplicates and
+    sort the data by timestamp to store it in to a copy of the table.
+
+    """
+    table_name = 'abstract_data'  # overwrite with 'weather' or 'singles'
+
+    def process_and_store_results(self, destination=None, overwrite=False, limit=None):
+        """Process table and store the results.
+
+        :param destination: name of the table where the results will be
+            written.  The default, None, corresponds to the value stored
+            in self.table_name.
+        :param overwrite: if True, overwrite previously obtained results.
+        :param limit: the maximum number of records that will be stored.
+            The default, None, corresponds to no limit.
+
+        """
+        self.limit = limit
+
+        self._check_destination(destination, overwrite)
+
+        self._clean_data_table()
+
+    def _get_source(self, source):
+        """Return the table containing the events.
+
+        :param source: the *name* of the table.  If None, this method will
+            try to find the original table.
+        :return: table object
+
+        """
+        if source is None:
+            source = self.data.get_node(self.group, self.table_name)
+        else:
+            source = self.data.get_node(self.group, source)
+        return source
+
+    def _check_destination(self, destination, overwrite):
+        """Check if the destination is valid"""
+
+        if destination == f'_t_{self.table_name}':
+            raise RuntimeError(f"The _t_{self.table_name} table is for internal use. Choose "
+                               "another destination.")
+        elif destination is None:
+            destination = self.table_name
+
+        # If destination == source, source will be overwritten.
+        if self.source.name != destination:
+            if destination in self.group and not overwrite:
+                raise RuntimeError("I will not overwrite previous results "
+                                   "(unless you specify overwrite=True)")
+
+        self.destination = destination
+
+    def _clean_data_table(self):
+        """Clean the table.
+
+        Remove duplicate events and sort the table by timestamp.
+
+        """
+        data = self.source
+
+        enumerated_timestamps = list(enumerate(data.col('timestamp')))
+        enumerated_timestamps.sort(key=operator.itemgetter(1))
+
+        unique_sorted_ids = self._find_unique_row_ids(enumerated_timestamps)
+
+        new_data = self._replace_table_with_selected_rows(data,
+                                                          unique_sorted_ids)
+        self.source = new_data
+        self._normalize_event_ids(new_data)
+
+    def _replace_table_with_selected_rows(self, table, row_ids):
+        """Replace data table with selected rows.
+
+        :param table: original table to be replaced.
+        :param row_ids: row ids of the selected rows which should go in
+            the destination table.
+
+        """
+        tmptable = self.data.create_table(self.group,
+                                          f'_t_{self.table_name}',
+                                          description=table.description)
+        selected_rows = table.read_coordinates(row_ids)
+        tmptable.append(selected_rows)
+        tmptable.flush()
+        self.data.rename_node(tmptable, self.destination, overwrite=True)
+        return tmptable
+
+
+class ProcessDataTableFromSource(ProcessDataTable):
+
+    """Process HiSPARC abstract data table from a different source.
+
+    This class is a subclass of ProcessDataTable. The difference is that in
+    this class, the source and destination are assumed to be different
+    files.  This also means that the source is untouched (no renaming of
+    original event tables) and the destination is assumed to be empty.
+
+    """
+
+    def __init__(self, source_file, dest_file, source_group, dest_group, progress=False):
+        """Initialize the class.
+
+        :param source_file,dest_file: PyTables source and destination files.
+        :param source_group,dest_group: the pathname of the source and
+                                        destination group.
+        :param progress: if True show a progressbar while copying and
+                         processing events.
+
+        """
+        self.source_file = source_file
+        self.dest_file = dest_file
+
+        self.source_group = self.source_file.get_node(source_group)
+        self.dest_group = self.dest_file.get_node(dest_group)
+
+        self.source = self._get_source()
+
+        self.progress = progress
+
+    def _get_source(self):
+        """Return the table containing the events.
+
+        :return: table object
+
+        """
+        source = self.source_file.get_node(self.source_group, self.table_name)
+        return source
+
+    def _check_destination(self, destination, overwrite):
+        """Override method, the destination should be empty"""
+        pass
+
+    def _replace_table_with_selected_rows(self, table, row_ids):
+        """Replace data table with selected rows.
+
+        :param table: original table to be replaced.
+        :param row_ids: row ids of the selected rows which should go in
+            the destination table.
+
+        """
+        new_table = self.dest_file.create_table(self.dest_group,
+                                                self.table_name,
+                                                description=table.description)
+        selected_rows = table.read_coordinates(row_ids)
+        new_table.append(selected_rows)
+        new_table.flush()
+        return new_table
+
+    def __repr__(self):
+        if not self.source_file.isopen or not self.dest_file.isopen:
+            return "<finished %s>" % self.__class__.__name__
+        else:
+            return ("%s(%r, %r, %r, %r, progress=%r)" %
+                    (self.__class__.__name__, self.source_file.filename,
+                     self.dest_file.filename, self.source_group._v_pathname,
+                     self.dest_group._v_pathname, self.progress))
+
+
+class ProcessWeather(ProcessDataTable):
+
+    """Process HiSPARC weather to clean the data.
+
+    This class can be used to process a set of HiSPARC weather, to
+    remove duplicates and sort the data by timestamp to store it in to a
+    copy of the weather table.
+
+    """
+    table_name = 'weather'
+
+
+class ProcessWeatherFromSource(ProcessDataTableFromSource):
+
+    """Process HiSPARC weather from a different source.
+
+    This class behaves like a subclass of ProcessWeather because of a common
+    ancestor (ProcessDataTable). The difference between this class
+    and ProcessWeather is that in this class, the source and destination are
+    assumed to be different files. This also means that the source is
+    untouched (no renaming of original event tables) and the destination is
+    assumed to be empty.
+
+    """
+    table_name = 'weather'
+
+
+class ProcessSingles(ProcessDataTable):
+
+    """Process HiSPARC singles data to clean the data.
+
+    This class can be used to process a set of HiSPARC singles data, to
+    remove duplicates and sort the data by timestamp to store it in to a
+    copy of the singles data table.
+
+    """
+    table_name = 'singles'
+
+
+class ProcessSinglesFromSource(ProcessDataTableFromSource):
+
+    """Process HiSPARC singles data from a different source.
+
+    This class behaves like a subclass of ProcessSingles because of a common
+    ancestor (ProcessDataTable). The difference between this class
+    and ProcessSingles is that in this class, the source and destination are
+    assumed to be different files. This also means that the source is
+    untouched (no renaming of original event tables) and the destination is
+    assumed to be empty.
+
+    """
+    table_name = 'singles'
