@@ -4,22 +4,26 @@ Classes corresponding to CORSIKA blocks and sub-blocks
 The classes in this module correspond one-to-one with the sub-blocks
 (and blocks) as specified in the CORSIKA users manual.
 
-Author: Javier Gonzalez <jgonzalez@ik.fzk.de>
+Author: Javier Gonzalez
 """
 
-import textwrap
-import struct
 import math
+import struct
 
 import numpy
 
-import units
-import particles
+from . import particles, units
+
+try:
+    from numba import jit
+except ImportError:
+    def jit(func):
+        return func
 
 
 # All sizes are in bytes
 
-class Format(object):
+class Format:
 
     """The binary format information of the file.
 
@@ -47,14 +51,23 @@ class Format(object):
         # number of particle records
         # With the unthinned option, each of these is 7 fields long
         # for a total of 39 records per sub block
-        self.particle_format = '7f'
+        self.fields_per_particle = 7
+        self.particle_format = '%df' % self.fields_per_particle
         self.particle_size = struct.calcsize(self.particle_format)
         self.particles_per_subblock = 39
 
+        # Full particle sub block
+        self.particles_format = (self.particle_format *
+                                 self.particles_per_subblock)
+        self.particles_size = self.particle_size * self.particles_per_subblock
+
+    def __repr__(self):
+        return '%s()' % self.__class__.__name__
+
 
 # From here on, things should not depend on the field size as everything is
 
-class RunHeader(object):
+class RunHeader:
 
     """The run header sub-block
 
@@ -117,13 +130,15 @@ class RunHeader(object):
         """
         a, b, c = self.a_atmospheric, self.b_atmospheric, self.c_atmospheric
 
-        if thickness > self.height_to_thickness(4.e5 * units.cm):
+        layers = [layer * units.cm for layer in self.atmospheric_layer_boundaries]
+
+        if thickness > self.height_to_thickness(layers[1]):
             height = c[0] * math.log(b[0] / (thickness - a[0]))
-        elif thickness > self.height_to_thickness(1.e6 * units.cm):
+        elif thickness > self.height_to_thickness(layers[2]):
             height = c[1] * math.log(b[1] / (thickness - a[1]))
-        elif thickness > self.height_to_thickness(4.e6 * units.cm):
+        elif thickness > self.height_to_thickness(layers[3]):
             height = c[2] * math.log(b[2] / (thickness - a[2]))
-        elif thickness > self.height_to_thickness(1.e7 * units.cm):
+        elif thickness > self.height_to_thickness(layers[4]):
             height = c[3] * math.log(b[3] / (thickness - a[3]))
         else:
             height = (a[4] - thickness) * c[4] / b[4]
@@ -139,21 +154,26 @@ class RunHeader(object):
         height = height * units.m / units.cm
         a, b, c = self.a_atmospheric, self.b_atmospheric, self.c_atmospheric
 
-        if height < 4.e5:
+        if height < self.atmospheric_layer_boundaries[1]:
+            # 0-4 km
             thickness = a[0] + b[0] * math.exp(-height / c[0])
-        elif height < 1.e6:
+        elif height < self.atmospheric_layer_boundaries[2]:
+            # 4-10 km
             thickness = a[1] + b[1] * math.exp(-height / c[1])
-        elif height < 4.e6:
+        elif height < self.atmospheric_layer_boundaries[3]:
+            # 10-40 km
             thickness = a[2] + b[2] * math.exp(-height / c[2])
-        elif height < 1.e7:
+        elif height < self.atmospheric_layer_boundaries[4]:
+            # 40-100 km
             thickness = a[3] + b[3] * math.exp(-height / c[3])
         else:
+            # >100 km
             thickness = a[4] - b[4] * height / c[4]
 
         return thickness
 
 
-class EventHeader(object):
+class EventHeader:
 
     """The event header sub-block
 
@@ -192,9 +212,9 @@ class EventHeader(object):
             self.azimuth = azimuth
 
         self.n_seeds = subblock[12]
-        self.seeds = numpy.array(zip(subblock[13:41:3],
-                                     subblock[14:42:3],
-                                     subblock[15:43:3]))
+        self.seeds = numpy.array(list(zip(subblock[13:41:3],
+                                          subblock[14:42:3],
+                                          subblock[15:43:3])))
 
         self.run_number = subblock[43]
         self.date_start = subblock[44]
@@ -309,8 +329,15 @@ class EventHeader(object):
         computers = {3: 'UNIX', 4: 'Macintosh'}
         return computers.get(self.flag_computer, 'unknown')
 
+    def __repr__(self):
+        return ('<%s, particle: %r, energy: 10**%.1f eV, zenith: %r deg,'
+                ' azimuth: %r deg>' %
+                (self.__class__.__name__, self.particle,
+                 math.log10(self.energy), math.degrees(self.zenith),
+                 math.degrees(self.azimuth)))
 
-class RunEnd(object):
+
+class RunEnd:
 
     """The run end sub-block
 
@@ -323,8 +350,16 @@ class RunEnd(object):
         self.run_number = subblock[1]
         self.n_events_processed = subblock[2]
 
+    def __repr__(self):
+        return '{}(({!r}, {!r}, {!r}))'.format(
+            self.__class__.__name__,
+            self.id,
+            self.run_number,
+            self.n_events_processed
+        )
 
-class EventEnd(object):
+
+class EventEnd:
 
     """The event end sub-block
 
@@ -375,6 +410,7 @@ class EventEnd(object):
         self.n_preshower_EM_particles = subblock[266]
 
 
+@jit
 def particle_data(subblock):
     """Get particle data.
 
@@ -385,8 +421,8 @@ def particle_data(subblock):
     Transformations are needed for the x, y and p_z values from CORSIKA.
     CORSIKA coordinate conventions are mentioned in Figure 1 and Table 10.
 
-    :returns: tuple with p_x, p_y, p_z, x, y, t, id, r, hadron_generation,
-              observation_level, phi data.
+    :return: tuple with p_x, p_y, p_z, x, y, t, id, r, hadron_generation,
+             observation_level, phi data.
 
     """
     # These three are subject to coordinate transformations
@@ -402,8 +438,8 @@ def particle_data(subblock):
     y = x_corsika
     t = subblock[6] * units.ns  # or z for additional muon info
 
-    id = description / 1000
-    hadron_generation = description / 10 % 100
+    id = description // 1000
+    hadron_generation = description // 10 % 100
     observation_level = description % 10
 
     r = math.sqrt(x ** 2 + y ** 2)
@@ -413,7 +449,20 @@ def particle_data(subblock):
             observation_level, phi)
 
 
-class ParticleData(object):
+@jit
+def particle_data_thin(subblock):
+    """Get thinned particle data.
+
+    Similar to :func:`particle_data`, but includes particle weight.
+
+    :return: tuple with p_x, p_y, p_z, x, y, t, id, r, hadron_generation,
+             observation_level, phi, weight data.
+
+    """
+    return particle_data(subblock[:7]) + (subblock[7],)
+
+
+class ParticleData:
 
     """The particle data sub-block
 
@@ -448,7 +497,7 @@ class ParticleData(object):
         return 200 <= self.id < 9900 or self.id == 14
 
     @property
-    def is_Cherenkov(self):
+    def is_cherenkov(self):
         return 9900 <= self.id
 
     @property
@@ -468,8 +517,13 @@ class ParticleData(object):
         else:
             return None
 
+    def __repr__(self):
+        return ('<%s, particle: %r, x: %r m, y: %r m, t: %r ns>' %
+                (self.__class__.__name__, self.particle, self.x, self.y,
+                 self.t))
 
-class CherenkovData(object):
+
+class CherenkovData:
 
     """The cherenkov photon sub-block
 
@@ -501,7 +555,7 @@ class FormatThin(Format):
     """
 
     def __init__(self):
-        super(FormatThin, self).__init__()
+        super().__init__()
 
         # one block contains 6552 fields plus one header and one end field
         self.block_format = '6554f'
@@ -517,9 +571,15 @@ class FormatThin(Format):
         # number of particle records
         # With the thinned option, each of these is 8 fields long
         # for a total of 39 records per sub block
-        self.particle_format = '8f'
+        self.fields_per_particle = 8
+        self.particle_format = '%df' % self.fields_per_particle
         self.particle_size = struct.calcsize(self.particle_format)
 
+        # Full particle sub block
+        self.particles_format = (self.particle_format *
+                                 self.particles_per_subblock)
+        self.particles_size = self.particle_size * self.particles_per_subblock
+
 
 class ParticleDataThin(ParticleData):
 
@@ -530,8 +590,9 @@ class ParticleDataThin(ParticleData):
     """
 
     def __init__(self, subblock):
-        self.weight = subblock[7]
-        super(ParticleDataThin, self).__init__(subblock)
+        self.p_x, self.p_y, self.p_z, self.x, self.y, self.t, self.id, \
+            self.r, self.hadron_generation, self.observation_level, \
+            self.phi, self.weight = particle_data_thin(subblock)
 
 
 class CherenkovDataThin(CherenkovData):
@@ -547,4 +608,4 @@ class CherenkovDataThin(CherenkovData):
 
     def __init__(self, subblock):
         self.weight = subblock[7]
-        super(CherenkovDataThin, self).__init__(subblock)
+        super().__init__(subblock)
