@@ -1,4 +1,4 @@
-""" Generate and overview table of the CORSIKA simulations
+""" Generate an overview table of the CORSIKA simulations
 
     This script will look for all completed and converted CORSIKA
     simulations in the given data path. Information about each
@@ -6,27 +6,27 @@
     overview.
 
     The given source path should contain subdirectories named after the
-    seeds used for the simulation in the format `{seed1}_{seed2}`, e.g.
-    `821280921_182096636`. These in turn should contain converted
-    CORSIKA simulation results called `corsika.h5`.
+    seeds used for the simulation in the format ``{seed1}_{seed2}``,
+    e.g. ``821280921_182096636``. These in turn should contain converted
+    CORSIKA simulation results called ``corsika.h5``.
 
 """
-import os
-import tables
+import argparse
 import glob
 import logging
+import os
 import shutil
-import argparse
 import tempfile
 
-from ..utils import pbar
+import tables
 
+from ..utils import pbar
 
 LOGFILE = '/data/hisparc/corsika/logs/generate_overview.log'
 DATA_PATH = '/data/hisparc/corsika/data'
 OUTPUT_PATH = '/data/hisparc/corsika/corsika_overview.h5'
 
-logger = logging.getLogger('generate_corsika_overview')
+logger = logging.getLogger(__name__)
 
 
 class Simulations(tables.IsDescription):
@@ -86,16 +86,19 @@ def read_seeds(simulations_table, source, seeds):
     :param seeds: directory name of a simulation, format: '{seed1}_{seed2}'.
 
     """
+    path = os.path.join(source, seeds, 'corsika.h5')
+    if not os.path.exists(path):
+        logger.info('%19s: No corsika.h5 available.' % seeds)
+        return
     try:
-        with tables.open_file(os.path.join(source, seeds, 'corsika.h5'),
-                              'r') as corsika_data:
+        with tables.open_file(path, 'r') as corsika_data:
             try:
                 header = corsika_data.get_node_attr('/', 'event_header')
                 end = corsika_data.get_node_attr('/', 'event_end')
                 write_row(simulations_table, seeds, header, end)
             except AttributeError:
                 logger.info('%19s: Missing attribute (header or end).' % seeds)
-    except (IOError, tables.HDF5ExtError):
+    except (OSError, tables.HDF5ExtError):
         logger.info('%19s: Unable to open file.' % seeds)
 
 
@@ -103,8 +106,7 @@ def get_simulations(source, simulations, overview, progress=False):
     """Get the information of the simulations and create a table."""
 
     simulations_table = overview.get_node('/simulations')
-    simulations = pbar(simulations, show=progress)
-    for seeds in simulations:
+    for seeds in pbar(simulations, show=progress):
         read_seeds(simulations_table, source, seeds)
     simulations_table.flush()
 
@@ -113,10 +115,10 @@ def prepare_output(n):
     """Create a temporary file in which to store the overview
 
     :param n: the number of simulations, i.e. expected number of rows.
-    :returns: path to the temporary file and a PyTables handler for the file.
+    :return: path to the temporary file and a PyTables handler for the file.
 
     """
-    os.umask(002)
+    os.umask(0o02)
     tmp_path = create_tempfile_path()
     overview = tables.open_file(tmp_path, 'w')
     overview.create_table('/', 'simulations', Simulations,
@@ -134,10 +136,25 @@ def move_tempfile_to_destination(tmp_path, destination):
     shutil.move(tmp_path, destination)
 
 
+def all_seeds(source):
+    """Get set of all seeds in the corsika data directory"""
+
+    dirs = glob.glob(os.path.join(source, '*_*'))
+    seeds = [os.path.basename(dir) for dir in dirs]
+    return sorted(set(seeds))
+
+
 def generate_corsika_overview(source, destination, progress=False):
+    """Create an overview of CORSIKA simulations for use by CorsikaQuery
+
+    :param source: directory containing the CORSIKA simulations.
+    :param destination: path of the HDF5 output file.
+    :param progress: if True show a progressbar while processing simulations.
+
+    """
     logger.info('Getting simulation list.')
     # Get names of all subdirectories
-    simulations = os.walk(source).next()[1]
+    simulations = all_seeds(source)
     tmp_path, overview = prepare_output(len(simulations))
     get_simulations(source, simulations, overview, progress=progress)
     overview.close()
@@ -146,7 +163,8 @@ def generate_corsika_overview(source, destination, progress=False):
 
 
 def main():
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(description='Generate an overview of '
+                                                 'CORSIKA simulations.')
     parser.add_argument('source', nargs='?', default=DATA_PATH,
                         help="directory path containing CORSIKA simulations")
     parser.add_argument('destination', nargs='?', default=OUTPUT_PATH,
@@ -154,13 +172,22 @@ def main():
     parser.add_argument('--progress', action='store_true',
                         help='show progressbar during generation')
     parser.add_argument('--log', action='store_true',
-                        help='write logs to file, only for use on server.')
+                        help='write logs to file, only for use on server')
+    parser.add_argument('--lazy', action='store_true',
+                        help='only run if the overview is outdated')
     args = parser.parse_args()
     if args.log:
         logging.basicConfig(filename=LOGFILE, filemode='a',
                             format='%(asctime)s %(name)s %(levelname)s: '
                                    '%(message)s',
                             datefmt='%y%m%d_%H%M%S', level=logging.INFO)
+    if args.lazy:
+        last_store = os.path.getmtime(args.source)
+        last_overview = os.path.getmtime(args.destination)
+        if last_overview > last_store:
+            logger.info('Overview up to date.')
+            return
+
     generate_corsika_overview(source=args.source,
                               destination=args.destination,
                               progress=args.progress)
