@@ -1,30 +1,45 @@
 """ Run CORSIKA simulations on Stoomboot
 
-    This module submits CORSIKA simulation jobs to Stoomboot, the Nikhef
-    computer cluster. It ensures that a unique combination of seeds for
-    the random number sequences are used for each simulation.
+    In order to quickly get a good sample of simulated showers we use the
+    Nikhef computer cluster Stoomboot to run multiple jobs simultaneously.
+    For this purpose a script has been written that will make this easy.
+    The :mod:`~sapphire.corsika.qsub_corsika` script can submit as many
+    jobs as you want with the parameters that you desire. It automatically
+    ensures that a unique combination of seeds for the random number
+    sequences are used for each simulation.
 
     To run this file correctly do it in the correct env::
 
-        source activate /data/hisparc/corsika_env
-        qsub_corsika 10 16 proton 22.5 -q generic -a 90
+        $ source activate corsika
+
+    The syntax for calling the script can be seen by calling its help::
+
+        $ qsub_corsika --help
+
+    For example, running 100 showers with proton primaries of 1e16 eV
+    coming in at 22.5 degrees zenith and 90 degrees azimuth on the
+    standard Stoomboot queue with the default CORSIKA configuration::
+
+        $ qsub_corsika 100 16 proton 22.5 -q generic -a 90
 
 """
-import os
-import sys
-import shutil
-import random
-import textwrap
-import subprocess
+
 import argparse
-from math import modf
+import os
+import random
+import subprocess
+import textwrap
+import warnings
 
-from sapphire.corsika import particles
-from sapphire.utils import pbar
+from math import log10, modf
 
+from .. import qsub
+from ..utils import pbar
+from . import particles
 
 TEMPDIR = '/data/hisparc/corsika/running/'
 DATADIR = '/data/hisparc/corsika/data/'
+FAILDIR = '/data/hisparc/corsika/failed/'
 CORSIKADIR = '/data/hisparc/corsika/corsika-74000/run/'
 INPUT_TEMPLATE = textwrap.dedent("""\
     RUNNR     0                        run number
@@ -59,37 +74,27 @@ INPUT_TEMPLATE = textwrap.dedent("""\
     DIRECT    ./                       output directory
     USER      hisparc                  user
     DEBUG     F  6  F  1000000         debug flag and log.unit for out
-    EXIT                               terminates input""")
+    EXIT                               terminates input""")  # noqa: E501
 SCRIPT_TEMPLATE = textwrap.dedent("""\
     #!/usr/bin/env bash
 
     umask 002
 
-    NAME="his_{seed1}_{seed2}"
-    SCRIPT=/tmp/$NAME
-
-    # Start Stoomboot script
-    cat >> $SCRIPT << EOF
-    #!/usr/bin/env bash
-
-    umask 002
-    export PATH=\${{PBS_O_PATH}}
-
     # Run CORSIKA
     /usr/bin/time -o time.log {corsika} < input-hisparc > corsika-output.log
 
     # Clean up after run
-    mv {rundir} {datadir}
-
-    EOF
-    # End of Stoomboot script
-
-    chmod ug+x $SCRIPT
-    qsub -N ${{NAME}} -q {queue} {walltime} -V -z -j oe -d {rundir} $SCRIPT
-    rm $SCRIPT""")
+    if [ $? -eq 0 ]
+    then
+        mv {rundir} {datadir}
+        exit 0
+    else
+        mv {rundir} {faildir}
+        exit 1
+    fi""")
 
 
-class CorsikaBatch(object):
+class CorsikaBatch:
 
     """Run many simultaneous CORSIKA simulations using Stoomboot
 
@@ -118,16 +123,8 @@ class CorsikaBatch(object):
 
     def __init__(self, energy=16, particle='proton', zenith=22.5, azimuth=180,
                  queue='generic', corsika='corsika74000Linux_QGSII_gheisha'):
-        # Energy is stored as log10(E[GeV]) for CORSIKA
-        if modf(energy)[0] == 0.:
-            self.energy_pre = 1.
-            self.energy_pow = int(energy - 9)
-        elif modf(energy)[0] == 0.5:
-            self.energy_pre = 3.16228
-            self.energy_pow = int(modf(energy)[1] - 9)
-        else:
-            raise ValueError('Energy must either be an integer or end in .5.')
-        self.particle = particles.particle_id(particle)  # Stored as particle id
+        self.energy_pre, self.energy_pow = self.corsika_energy(energy)
+        self.particle = particles.particle_id(particle)  # Store as particle id
         self.theta = zenith
         self.phi = (azimuth + 90) % 360  # Stored as Phi defined by CORSIKA
         self.queue = queue
@@ -136,6 +133,23 @@ class CorsikaBatch(object):
         self.seed2 = None
         self.rundir = None
 
+    def corsika_energy(self, energy):
+        """Convert energy to format for CORSIKA input
+
+        Convert from log10(E/eV) to E[GeV] and split value to multiplier
+        and power.
+
+        :param energy: primary particle energy as log10(E/eV).
+        :return: separate multiplier and power
+
+        """
+        if modf(energy)[0] == 0.:
+            return (1., int(energy - 9))
+        elif modf(energy)[0] == 0.5:
+            return (3.16228, int(modf(energy)[1] - 9))
+        else:
+            raise ValueError('Energy must either be an integer or end in .5.')
+
     def run(self):
         self.prepare_env()
         self.submit_job()
@@ -144,27 +158,28 @@ class CorsikaBatch(object):
         """Setup CORSIKA environment"""
 
         # Set umask
-        os.umask(002)
+        os.umask(0o02)
 
         # Setup directories
         taken = self.taken_seeds()
-        self.gen_random_seeds(taken)
+        self.generate_random_seeds(taken)
         self.make_rundir()
         self.goto_rundir()
 
         # Create/copy files
         self.create_input()
-        self.create_script()
         self.copy_config()
 
     def submit_job(self):
         """Submit job to Stoomboot"""
 
-        result = subprocess.check_output('./run.sh', stderr=subprocess.STDOUT,
-                                         shell=True)
-        if not result == '':
-            print '%sError occured: %s' % (self.rundir, result)
-            raise Exception
+        name = f"cor_{self.seed1}_{self.seed2}"
+        extra = f"-d {self.get_rundir()}"
+        if self.queue == 'long':
+            extra += " -l walltime=96:00:00"
+        script = self.create_script()
+
+        qsub.submit_job(script, name, self.queue, extra)
 
     def taken_seeds(self):
         """Get list of seeds already used"""
@@ -173,7 +188,7 @@ class CorsikaBatch(object):
         taken.extend(os.listdir(TEMPDIR))
         return taken
 
-    def gen_random_seeds(self, taken):
+    def generate_random_seeds(self, taken):
         """Get unused combination of two seeds for CORSIKA
 
         :param taken: List of seed combinations already taken
@@ -182,57 +197,50 @@ class CorsikaBatch(object):
         """
         seed1 = random.randint(1, 900000000)
         seed2 = random.randint(1, 900000000)
-        seed = "{seed1}_{seed2}".format(seed1=seed1, seed2=seed2)
-        if not seed in taken:
+        seed = f"{seed1}_{seed2}"
+        if seed not in taken:
             self.seed1 = seed1
             self.seed2 = seed2
-            self.rundir = seed +'/'
+            self.rundir = seed + '/'
         else:
-            self.get_random_seeds(taken)
+            self.generate_random_seeds(taken)
 
     def make_rundir(self):
         """Make the run directory"""
 
-        os.mkdir(os.path.join(TEMPDIR, self.rundir))
+        os.mkdir(self.get_rundir())
 
     def goto_rundir(self):
         """Move into the run directory"""
 
-        os.chdir(os.path.join(TEMPDIR + self.rundir))
+        os.chdir(self.get_rundir())
+
+    def get_rundir(self):
+        """Get run directory path"""
+
+        return os.path.join(TEMPDIR, self.rundir)
 
     def create_input(self):
         """Make CORSIKA steering file"""
 
-        inputpath = os.path.join(TEMPDIR, self.rundir, 'input-hisparc')
+        input_path = os.path.join(self.get_rundir(), 'input-hisparc')
         input = INPUT_TEMPLATE.format(seed1=self.seed1, seed2=self.seed2,
                                       particle=self.particle, phi=self.phi,
                                       energy_pre=self.energy_pre,
                                       energy_pow=self.energy_pow,
                                       theta=self.theta, tablesdir=CORSIKADIR)
-        file = open(inputpath, 'w')
-        file.write(input)
-        file.close()
+        with open(input_path, 'w') as input_file:
+            input_file.write(input)
 
     def create_script(self):
         """Make Stoomboot script file"""
 
-        if self.queue == 'long':
-            walltime = "-l walltime=96:00:00"
-        else:
-            walltime = ""
-
         exec_path = os.path.join(CORSIKADIR, self.corsika)
-        run_path = os.path.join(TEMPDIR, self.rundir)
-        script_path = os.path.join(run_path, 'run.sh')
+        run_path = self.get_rundir()
 
-        script = SCRIPT_TEMPLATE.format(seed1=self.seed1, seed2=self.seed2,
-                                        queue=self.queue, walltime=walltime,
-                                        corsika=exec_path, rundir=run_path,
-                                        datadir=DATADIR)
-        file = open(script_path, 'w')
-        file.write(script)
-        file.close()
-        os.chmod(script_path, 0774)
+        script = SCRIPT_TEMPLATE.format(corsika=exec_path, rundir=run_path,
+                                        datadir=DATADIR, faildir=FAILDIR)
+        return script
 
     def copy_config(self):
         """Copy the CORSIKA config file to the output directory
@@ -242,127 +250,91 @@ class CorsikaBatch(object):
 
         """
         source = os.path.join(CORSIKADIR, self.corsika + '.log')
-        destination = os.path.join(TEMPDIR, self.rundir)
-        subprocess.check_output('cp {source} {destination}'
-                                .format(source=source,
-                                        destination=destination),
-                                shell=True)
+        destination = self.get_rundir()
+        subprocess.check_output(['cp', source, destination])
 
-    def symlink_corsika(self):
-        """Create symbolic links to CORSIKA run files
-
-        No longer used, was used to symlink the CORSIKA executable
-        to the output directory.
-
-        """
-        source = os.path.join(CORSIKADIR, self.corsika)
-        destination = os.path.join(TEMPDIR, self.rundir)
-        subprocess.check_output('ln -s {source} {destination}'
-                                .format(source=source,
-                                        destination=destination),
-                                shell=True)
+    def __repr__(self):
+        energy = round(log10(self.energy_pre * 10 ** self.energy_pow) + 9, 1)
+        particle = particles.name(self.particle)
+        azimuth = self.phi - 90
+        return ('%s(energy=%r, particle=%r, zenith=%r, azimuth=%r, queue=%r, '
+                'corsika=%r)' %
+                (self.__class__.__name__, energy, particle, self.theta,
+                 azimuth, self.queue, self.corsika))
 
 
-def check_queue(queue):
-    """Check for available job slots on the selected queue for current user
-
-    Maximum numbers from `qstat -Q -f`
-
-    Caveat: also counts completed jobs still listed in qstat,
-            add `grep [RQ]` to fix.
-
-    :param queue: queue name for which to check current number of job
-                  slots in use.
-    :return: boolean, True if slots are available, False otherwise.
-
-    """
-    all_jobs = int(subprocess.check_output('qstat | grep {queue} | '
-                                           'grep " [QR] " | wc -l'
-                                           .format(queue=queue), shell=True))
-    user_jobs = int(subprocess.check_output('qstat -u $USER | grep {queue} | '
-                                            'grep " [QR] " | wc -l'
-                                            .format(queue=queue), shell=True))
-
-    if queue == 'express':
-        return 2 - user_jobs
-    elif queue == 'short':
-        return 1000 - user_jobs
-    elif queue == 'generic':
-        return min(2000 - user_jobs, 4000 - all_jobs)
-    elif queue == 'long':
-        return min(500 - user_jobs, 1000 - all_jobs)
-    else:
-        raise KeyError('Unknown queue name: {queue}'.format(queue))
-
-
-def multiple_jobs(n, energy, particle, zenith, azimuth, queue, corsika):
+def multiple_jobs(n, energy, particle, zenith, azimuth, queue, corsika,
+                  progress=True):
     """Use this to sumbit multiple jobs to Stoomboot
 
     :param n: Number of jobs to submit
     :param energy: log10(E[eV]) energy of primary particle
-    :param particle: Particle kind (as string, see particles.py for possibilities)
+    :param particle: Particle kind (as string, see
+                     :mod:`~sapphire.corsika.particles` for possibilities)
     :param zenith: Zenith angle in degrees of the primary particle
     :param azimuth: Azimuth angle in degrees of the primary particle
     :param queue: Stoomboot queue to submit to
     :param corsika: Name of the CORSIKA executable to use
+    :param progress: if True print an overview of the chosen paramters and
+                     show a progressbar of the job submission progress.
 
     """
-    print textwrap.dedent("""\
-        Batch submitting jobs to Stoomboot:
-        Number of jobs      {n}
-        Particle energy     10^{e} eV
-        Primary particle    {p}
-        Zenith angle        {z} degrees
-        Azimuth angle       {a} degrees
-        Stoomboot queue     {q}
-        CORSIKA executable  {c}
-        """.format(n=n, e=energy, p=particle, z=zenith, a=azimuth, q=queue,
-                   c=corsika))
+    if progress:
+        print(textwrap.dedent("""\
+            Batch submitting jobs to Stoomboot:
+            Number of jobs      {n}
+            Particle energy     10^{e} eV
+            Primary particle    {p}
+            Zenith angle        {z} degrees
+            Azimuth angle       {a} degrees
+            Stoomboot queue     {q}
+            CORSIKA executable  {c}
+            """.format(n=n, e=energy, p=particle, z=zenith, a=azimuth, q=queue,
+                       c=corsika)))
 
-    available_slots = check_queue(queue)
+    available_slots = qsub.check_queue(queue)
     if available_slots <= 0:
-        n = 0
-        print 'Submitting no jobs because queue is full.'
-        return
+        raise Exception('Submitting no jobs because selected queue is full.')
     elif available_slots < n:
         n = available_slots
-        print 'Submitting {n} jobs because queue is almost full.'.format(n=n)
+        warnings.warn('Submitting {n} jobs because queue almost full.'
+                      .format(n=n))
 
-    for _ in pbar(xrange(n)):
+    for _ in pbar(range(n), show=progress):
         batch = CorsikaBatch(energy=energy, particle=particle, zenith=zenith,
                              azimuth=azimuth, queue=queue, corsika=corsika)
         batch.run()
-    print 'Done.'
 
 
 def main():
-    parser = argparse.ArgumentParser()
-    parser.add_argument('n', type=int, help="Number of jobs to submit")
+    parser = argparse.ArgumentParser(description='Submit CORSIKA jobs to '
+                                                 'Stoomboot, only at Nikhef.')
+    parser.add_argument('n', type=int, help="number of jobs to submit")
     parser.add_argument('energy', metavar='energy', type=float,
-                        help="Energy of the primary particle in range 12..17 "
-                             "(log10(E[eV])), in steps of .5 .",
-                        choices=[12, 12.5, 13, 13.5, 14, 14.5, 15, 15.5,
-                                 16, 16.5, 17, 17.5, 18, 18.5])
-    parser.add_argument('particle', help="Primary particle kind (e.g. proton "
+                        help="energy of the primary particle in range 11..17, "
+                             "in steps of .5 (log10(E[eV]))",
+                        choices=[11, 11.5, 12, 12.5, 13, 13.5, 14, 14.5,
+                                 15, 15.5, 16, 16.5, 17, 17.5, 18, 18.5])
+    parser.add_argument('particle', help="primary particle kind (e.g. proton "
                                          "or iron)")
     parser.add_argument('zenith', metavar='zenith',
-                        help="Zenith angle of primary particle in range 0..60 "
-                             "steps of 7.5 [degrees]",
+                        help="zenith angle of primary particle in range 0..60,"
+                             " in steps of 7.5 [degrees]",
                         type=float,
                         choices=[0, 7.5, 15, 22.5, 30, 37.5, 45, 52.5, 60])
     parser.add_argument('-a', '--azimuth', metavar='angle',
-                        help="Azimuth angle of primary particle in range "
-                             "0..315 steps of 45 [degrees]",
+                        help="azimuth angle of primary particle in range "
+                             "0..315, in steps of 45 [degrees]",
                         type=int,
                         default=0,
                         choices=[0, 45, 90, 135, 180, 225, 270, 315])
     parser.add_argument('-q', '--queue', metavar='name',
-                        help="Name of the Stoomboot queue to use, choose from "
-                             "[express, short, generic, long]",
+                        help="name of the Stoomboot queue to use, choose from "
+                             "express, short, generic (default), and long",
                         default='generic',
                         choices=['express', 'short', 'generic', 'long'])
     parser.add_argument('-c', '--corsika', metavar='exec',
-                        help="Name of the CORSIKA executable to use",
+                        help="name of the CORSIKA executable to use",
                         default="corsika74000Linux_QGSII_gheisha")
     args = parser.parse_args()
 
