@@ -1,17 +1,99 @@
-from struct import unpack
+""" Read CORSIKA data files.
+
+    This provides functionality to read CORSIKA output
+    files with `Python <www.python.org>`_. It provides the following main
+    classes:
+
+    * :class:`~sapphire.corsika.reader.CorsikaFile`: The file class provides a
+      generator over all events in the file.
+    * :class:`~sapphire.corsika.reader.CorsikaEvent`: The event class that
+      provides a generator over all particles at ground.
+
+    and the following classes that correspond to the sub-blocks defined in
+    the CORSIKA manual:
+
+    * :class:`~sapphire.corsika.blocks.RunHeader`
+    * :class:`~sapphire.corsika.blocks.RunEnd`
+    * :class:`~sapphire.corsika.blocks.EventHeader`
+    * :class:`~sapphire.corsika.blocks.EventEnd`
+    * :class:`~sapphire.corsika.blocks.ParticleData`
+    * :class:`~sapphire.corsika.blocks.CherenkovData`
+
+    Additionally version for thinned showers are available:
+
+    * :class:`CorsikaFileThin`
+    * :class:`~sapphire.corsika.blocks.ParticleDataThin`
+    * :class:`~sapphire.corsika.blocks.CherenkovDataThin`
+
+
+    Issues
+    ======
+
+    This module does not handle platform dependent issues such as byte
+    ordering (endianness) and field size. This was the result of an
+    afternoon hack and has only been tested with files generated using
+    32 bit CORSIKA files on a linux system compiled with gfortran.
+
+    * **Field Size**: According to the CORSIKA user manual section 10.2
+      all quantities are written as single precision real numbers
+      independently of 32-bit or 64-bit, so each field in the file
+      should be 4 bytes long.
+    * **Endianness**: There is no check for byte ordering. It can be added
+      using Python's `struct module
+      <http://docs.python.org/library/struct.html#struct-alignment>`_.
+    * **Special Particles**: This module currently ignores all special
+      (book-keeping) particles like for muon additional information and
+      history.
+
+
+    More Info
+    =========
+
+    For short information on fortran unformatted binary files, take a look
+    at http://paulbourke.net/dataformats/reading/
+
+    For detailed information on the CORSIKA format, check the 'Outputs'
+    chapter in the CORSIKA user manual. In particular, check the 'Normal
+    Particle Output' section.
+
+
+    Authors
+    =======
+
+    - Javier Gonzalez
+    - Arne de Laat <adelaat@nikhef.nl>
+
+"""
+
+import os
 import warnings
 
-from blocks import (RunHeader, RunEnd, EventHeader, EventEnd,
-                    ParticleData, Format, ParticleDataThin, FormatThin,
-                    particle_data)
+from struct import unpack
+
+from .blocks import (
+    EventEnd,
+    EventHeader,
+    Format,
+    FormatThin,
+    ParticleData,
+    ParticleDataThin,
+    RunEnd,
+    RunHeader,
+    particle_data,
+    particle_data_thin,
+)
 
 
-class CorsikaEvent(object):
+class CorsikaEvent:
     def __init__(self, raw_file, header_index, end_index):
-        """CorsikaEvent constructor.
+        """CorsikaEvent constructor
 
         The user never calls this. The CorsikaFile does.
 
+        :param raw_file: :class:`CorsikaFile` object.
+        :param header_index: index where the event header starts.
+        :param end_index: index where the event end starts.
+
         """
         self._raw_file = raw_file
         self._header_index = header_index
@@ -59,30 +141,34 @@ class CorsikaEvent(object):
         """
         for sub_block_index in self._raw_file._subblocks_indices(
                 self._header_index, self._end_index):
-            for p in range(self.format.particles_per_subblock):
-                pos = sub_block_index + p * self.format.particle_size
-                particle = self._raw_file._get_particle_record_tuple(pos)
-                type = particle[6]  # particle type
-                level = particle[9]  # observation level
+            for particle in self._raw_file._get_particles(sub_block_index):
+                particle_type = particle[6]
+                observation_level = particle[9]
 
                 # skip padding, used to fill a subblock
-                if type == 0:
+                if particle_type == 0:
                     continue
                 # muon additional information
-                if type in [75, 76]:
-                    warnings.warn('Ignoring muon additional information.',
-                                  UserWarning)
+                if particle_type in [75, 76]:
+                    warnings.warn('Ignoring muon additional information.')
                     continue
                 # ignore all observation levels except for nr. 1
-                if level != 1:
-                    warnings.warn('Only observation level 1 will be read!',
-                                  UserWarning)
+                if observation_level != 1:
+                    warnings.warn('Only observation level 1 will be read!')
                     continue
 
                 yield particle
 
+    def __repr__(self):
+        return "{}({!r}, {!r}, {!r})".format(
+            self.__class__.__name__,
+            self._raw_file,
+            self._header_index,
+            self._end_index
+        )
 
-class CorsikaFile(object):
+
+class CorsikaFile:
 
     """CORSIKA output file handler
 
@@ -99,14 +185,25 @@ class CorsikaFile(object):
 
         """
         self._filename = filename
-        with open(filename, 'rb') as corsika_data:
-            self._contents = corsika_data.read()
+        self._size = os.path.getsize(self._filename)
+        self._file = open(self._filename, 'rb')
         self._header_index = None
         self._end_index = None
         self._header = None
         self._end = None
         self.format = Format()
 
+    def finish(self):
+        """Close the opened CORSIKA data file"""
+
+        self._file.close()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.finish()
+
     def check(self):
         """Check DAT file format
 
@@ -123,17 +220,17 @@ class CorsikaFile(object):
         field size.
 
         """
-        if len(self._contents) % self.format.block_size != 0:
+        if self._size % self.format.block_size != 0:
             raise Exception('File "{name}" does not have an integer number '
                             'of blocks!'.format(name=self._filename))
         block_size = self.format.block_size
         padding = self.format.block_padding_size
-        n_blocks = len(self._contents) / block_size
+        n_blocks = self._size // block_size
         for block in range(n_blocks):
-            a = unpack('i', self._contents[block * block_size:
-                                           block * block_size + padding])[0]
-            b = unpack('i', self._contents[(block + 1) * block_size - padding:
-                                           (block + 1) * block_size])[0]
+            self._file.seek(block * block_size)
+            a = unpack('i', self._file.read(padding))[0]
+            self._file.seek((block + 1) * block_size - padding)
+            b = unpack('i', self._file.read(padding))[0]
             if a != b:
                 raise Exception('Block #{block} is not right: ({head}, {tail})'
                                 .format(block=block, head=a, tail=b))
@@ -148,12 +245,13 @@ class CorsikaFile(object):
         """
         block_size = self.format.block_size
         subblock_size = self.format.subblock_size
-        n_blocks = len(self._contents) / block_size
-        for b in xrange(0, n_blocks * block_size, block_size):
-            for s in xrange(0, self.format.subblocks_per_block):
+        n_blocks = self._size / block_size
+        for b in range(0, n_blocks * block_size, block_size):
+            for s in range(0, self.format.subblocks_per_block):
                 pos = b + s * subblock_size + self.format.block_padding_size
+                self._file.seek(pos)
                 yield unpack(self.format.subblock_format,
-                             self._contents[pos:pos + subblock_size])
+                             self._file.read(subblock_size))
 
     def get_header(self):
         """Get the Run header
@@ -193,11 +291,11 @@ class CorsikaFile(object):
         """
         event_head = None
         for block in self._subblocks_indices():
-            endblocktag = block + self.format.field_size
-            tag = unpack('4s', self._contents[block:endblocktag])[0]
-            if tag == 'EVTH':
+            self._file.seek(block)
+            tag = unpack('4s', self._file.read(self.format.field_size))[0]
+            if tag == b'EVTH':
                 event_head = block
-            elif tag == 'EVTE':
+            elif tag == b'EVTE':
                 yield CorsikaEvent(self, event_head, block)
                 event_head = None
 
@@ -210,9 +308,9 @@ class CorsikaFile(object):
         """
         block_size = self.format.block_size
         subblock_size = self.format.subblock_size
-        n_blocks = len(self._contents) / block_size
-        for b in xrange(0, n_blocks * block_size, block_size):
-            for s in xrange(0, self.format.subblocks_per_block):
+        n_blocks = self._size // block_size
+        for b in range(0, n_blocks * block_size, block_size):
+            for s in range(0, self.format.subblocks_per_block):
                 pos = b + s * subblock_size + self.format.block_padding_size
                 if ((min_sub_block is not None and pos <= min_sub_block) or
                         (max_sub_block is not None and pos >= max_sub_block)):
@@ -225,11 +323,11 @@ class CorsikaFile(object):
         heads = []
         tails = []
         for block in self._subblocks_indices():
-            endblock = block + self.format.field_size
-            type = unpack('4s', self._contents[block:endblock])[0]
-            if type == 'EVTH':
+            self._file.seek(block)
+            tag = unpack('4s', self._file.read(self.format.field_size))[0]
+            if tag == b'EVTH':
                 heads.append(block)
-            elif type == 'EVTE':
+            elif tag == b'EVTE':
                 tails.append(block)
         return (heads, tails)
 
@@ -246,11 +344,11 @@ class CorsikaFile(object):
     def _get_run_indices(self):
         """Get the indices for the start of the run header and end"""
         for block in self._subblocks_indices():
-            endblock = block + self.format.field_size
-            type = unpack('4s', self._contents[block:endblock])[0]
-            if type == 'RUNH':
+            self._file.seek(block)
+            tag = unpack('4s', self._file.read(self.format.field_size))[0]
+            if tag == b'RUNH':
                 head = block
-            elif type == 'RUNE':
+            elif tag == b'RUNE':
                 tail = block
         return head, tail
 
@@ -274,15 +372,23 @@ class CorsikaFile(object):
 
         return particle_data(self._unpack_particle(word))
 
+    def _get_particles(self, word):
+        """Get subblock of particles from the contents as tuples"""
+
+        unpacked_particles = self._unpack_particles(word)
+        particles = zip(*[iter(unpacked_particles)] *
+                        self.format.fields_per_particle)
+        return (particle_data(particle) for particle in particles)
+
     def _unpack_subblock(self, word):
         """Unpack a subblock block
 
         :param word: the index where the subblock starts
 
         """
-        endword = word + self.format.subblock_size
+        self._file.seek(word)
         return unpack(self.format.subblock_format,
-                      self._contents[word:endword])
+                      self._file.read(self.format.subblock_size))
 
     def _unpack_particle(self, word):
         """Unpack a particle block
@@ -290,12 +396,22 @@ class CorsikaFile(object):
         :param word: the index where the particle subblock starts
 
         """
-        endword = word + self.format.particle_size
+        self._file.seek(word)
         return unpack(self.format.particle_format,
-                      self._contents[word:endword])
+                      self._file.read(self.format.particle_size))
 
-    def Blocks():
-        pass
+    def _unpack_particles(self, word):
+        """Unpack a particles subblock
+
+        :param word: the index where the particle subblock starts
+
+        """
+        self._file.seek(word)
+        return unpack(self.format.particles_format,
+                      self._file.read(self.format.particles_size))
+
+    def __repr__(self):
+        return f"{self.__class__.__name__}({self._filename!r})"
 
 
 class CorsikaFileThin(CorsikaFile):
@@ -314,10 +430,23 @@ class CorsikaFileThin(CorsikaFile):
         It takes a filename as argument
 
         """
-        super(CorsikaFileThin, self).__init__(filename)
+        super().__init__(filename)
         self.format = FormatThin()
 
     def _get_particle_record(self, word):
         """Get a particle from the contents as a ParticleDataThin instance"""
 
         return ParticleDataThin(self._unpack_particle(word))
+
+    def _get_particle_record_tuple(self, word):
+        """Get a thinned particle from the contents as a tuple"""
+
+        return particle_data_thin(self._unpack_particle(word))
+
+    def _get_particles(self, word):
+        """Get subblock of thinned particles from the contents as tuples"""
+
+        unpacked_particles = self._unpack_particles(word)
+        particles = zip(*[iter(unpacked_particles)] *
+                        self.format.fields_per_particle)
+        return (particle_data_thin(particle) for particle in particles)
