@@ -1,25 +1,30 @@
-""" Fetch events and other data from the public database
+""" Fetch raw events and other data from the public database
 
     This module enables you to access the public database and even the raw
-    event data.
-
-    For convenience, you'll want the :func:`download_data` function.
+    event data. This is intended for specialized use only. For most uses, it is
+    faster and more convenient to access the event summary data (ESD) using
+    :mod:`~sapphire.esd`.
 
 """
-import xmlrpclib
-import urllib
 import datetime
-import tables
+import logging
 import os
-import calendar
 import re
 
-import logging
+from urllib.parse import urljoin
+from urllib.request import urlretrieve
+from xmlrpc.client import ServerProxy
 
-logger = logging.getLogger('hisparc.publicdb')
+import tables
 
-# PUBLICDB_XMLRPC_URL = 'http://localhost:8000/raw_data/rpc'
-PUBLICDB_XMLRPC_URL = 'http://data.hisparc.nl/raw_data/rpc'
+from .transformations.clock import datetime_to_gps
+from .utils import get_publicdb_base
+
+logger = logging.getLogger(__name__)
+
+
+def get_publicdb_xmlrpc_url():
+    return urljoin(get_publicdb_base(), 'raw_data/rpc')
 
 
 def download_data(file, group, station_id, start, end, get_blobs=False):
@@ -47,28 +52,28 @@ def download_data(file, group, station_id, start, end, get_blobs=False):
         >>> start = datetime.datetime(2010, 9, 1)
         >>> end = datetime.datetime(2010, 9, 2)
         >>> sapphire.publicdb.download_data(data, '/s501', 501, start, end)
-        INFO:hisparc.publicdb:2010-09-01 00:00:00 None
-        INFO:hisparc.publicdb:Getting server data URL (2010-09-01 00:00:00)
-        INFO:hisparc.publicdb:Downloading data...
-        INFO:hisparc.publicdb:Storing data...
-        INFO:hisparc.publicdb:Done.
+        INFO:sapphire.publicdb:2010-09-01 00:00:00 None
+        INFO:sapphire.publicdb:Getting server data URL (2010-09-01 00:00:00)
+        INFO:sapphire.publicdb:Downloading data...
+        INFO:sapphire.publicdb:Storing data...
+        INFO:sapphire.publicdb:Done.
 
     """
-    server = xmlrpclib.ServerProxy(PUBLICDB_XMLRPC_URL)
+    server = ServerProxy(get_publicdb_xmlrpc_url())
 
     for t0, t1 in datetimerange(start, end):
-        logger.info("%s %s" % (t0, t1))
-        logger.info("Getting server data URL (%s)" % t0)
+        logger.info(f"{t0} {t1}")
+        logger.info(f"Getting server data URL {t0}")
         try:
             url = server.hisparc.get_data_url(station_id, t0, get_blobs)
-        except Exception, exc:
+        except Exception as exc:
             if re.search("No data", str(exc)):
-                logger.warning("No data for %s" % t0)
+                logger.warning(f"No data for {t0}")
                 continue
             else:
                 raise
         logger.info("Downloading data...")
-        tmp_datafile, headers = urllib.urlretrieve(url)
+        tmp_datafile, headers = urlretrieve(url)
         logger.info("Storing data...")
         _store_data(file, group, tmp_datafile, t0, t1)
         logger.info("Done.")
@@ -80,11 +85,8 @@ def _store_data(dst_file, dst_group, src_filename, t0, t1):
     This function takes a file containing downloaded data and copies it to
     the destination file, based on start and end timestamps.
 
-    This can be further optimized at the expense of spaghetti code.
-
     """
-    # Open in rw mode, need to update blob idxs, if necessary
-    with tables.open_file(src_filename, 'a') as src_file:
+    with tables.open_file(src_filename, 'r') as src_file:
         src_group = src_file.list_nodes('/')[0]
         dst_group = _get_or_create_group(dst_file, dst_group)
 
@@ -100,36 +102,32 @@ def _store_data(dst_file, dst_group, src_filename, t0, t1):
                 for row in node:
                     dst_node.append(row)
 
-            elif node.name == 'events':
-                if not t1:
-                    cond = 'timestamp >= %d' % \
-                        calendar.timegm(t0.utctimetuple())
+            elif node.name in ['events', 'errors', 'config', 'comparator',
+                               'singles', 'satellites', 'weather',
+                               'weather_error', 'weather_config']:
+                if t1 is None:
+                    cond = 'timestamp >= %d' % datetime_to_gps(t0)
                 else:
-                    cond = '(%d <= timestamp) & (timestamp <= %d)' % \
-                        (calendar.timegm(t0.utctimetuple()),
-                         calendar.timegm(t1.utctimetuple()))
+                    cond = ('(%d <= timestamp) & (timestamp <= %d)' %
+                            (datetime_to_gps(t0), datetime_to_gps(t1)))
+
+                rows = node.read_where(cond)
 
                 if len_blobs:
-                    for row in node.read_where(cond):
-                        row['traces'] += len_blobs
-                        dst_node.append([tuple(row)])
-                else:
-                    for row in node.read_where(cond):
-                        dst_node.append([tuple(row)])
-
-            elif node.name == 'errors' and len_blobs:
-                for row in node:
-                    row['messages'] += len_blobs
-                    dst_node.append([row[:]])
-
-            elif node.name == 'config' and len_blobs:
-                for row in node:
-                    row['mas_version'] += len_blobs
-                    row['slv_version'] += len_blobs
-                    row['password'] += len_blobs
-                    row['buffer'] += len_blobs
-                    dst_node.append([row[:]])
+                    if node.name == 'events':
+                        rows['traces'] += len_blobs
+                    elif node.name in ['errors', 'weather_error']:
+                        rows['messages'] += len_blobs
+                    elif node.name == 'config':
+                        rows['mas_version'] += len_blobs
+                        rows['slv_version'] += len_blobs
+                        rows['password'] += len_blobs
+                        rows['buffer'] += len_blobs
+                    elif node.name == 'weather_config':
+                        rows['help_url'] += len_blobs
+                        rows['database_name'] += len_blobs
 
+                dst_node.append(rows)
             else:
                 rows = node.read()
                 dst_node.append(rows)
@@ -151,46 +149,40 @@ def datetimerange(start, stop):
 
     Example::
 
-        >>> for x in datetimerange(datetime.datetime(2010, 1, 1, 11),
-        ...                        datetime.datetime(2010, 1, 1, 13)):
+        >>> for x in datetimerange(datetime.datetime(2010, 1, 1, 11), datetime.datetime(2010, 1, 1, 13)):
         ...     x
         ...
-        (datetime.datetime(2010, 1, 1, 11, 0),
-         datetime.datetime(2010, 1, 1, 13, 0))
+        (datetime.datetime(2010, 1, 1, 11, 0), datetime.datetime(2010, 1, 1, 13, 0))
 
-        >>> for x in datetimerange(datetime.datetime(2010, 1, 1, 11),
-        ...                        datetime.datetime(2010, 1, 2)):
+        >>> for x in datetimerange(datetime.datetime(2010, 1, 1, 11), datetime.datetime(2010, 1, 2)):
         ...     x
         ...
         (datetime.datetime(2010, 1, 1, 11, 0), None)
 
-        >>> for x in datetimerange(datetime.datetime(2010, 1, 1, 11),
-        ...                        datetime.datetime(2010, 1, 2, 13)):
+        >>> for x in datetimerange(datetime.datetime(2010, 1, 1, 11), datetime.datetime(2010, 1, 2, 13)):
         ...     x
         ...
         (datetime.datetime(2010, 1, 1, 11, 0), None)
-        (datetime.datetime(2010, 1, 2, 0, 0),
-         datetime.datetime(2010, 1, 2, 13, 0))
+        (datetime.datetime(2010, 1, 2, 0, 0), datetime.datetime(2010, 1, 2, 13, 0))
 
-        >>> for x in datetimerange(datetime.datetime(2010, 1, 1, 11),
-        ...                        datetime.datetime(2010, 1, 5, 13)):
+        >>> for x in datetimerange(datetime.datetime(2010, 1, 1, 11), datetime.datetime(2010, 1, 5, 13)):
         ...     x
         ...
         (datetime.datetime(2010, 1, 1, 11, 0), None)
         (datetime.datetime(2010, 1, 2, 0, 0), None)
         (datetime.datetime(2010, 1, 3, 0, 0), None)
         (datetime.datetime(2010, 1, 4, 0, 0), None)
-        (datetime.datetime(2010, 1, 5, 0, 0),
-         datetime.datetime(2010, 1, 5, 13, 0))
+        (datetime.datetime(2010, 1, 5, 0, 0), datetime.datetime(2010, 1, 5, 13, 0))
 
     """
-    if start.date() == stop.date():
+    if start > stop:
+        raise Exception('Start can not be after stop.')
+    elif start.date() == stop.date():
         yield start, stop
         return
     else:
         yield start, None
-        cur = (start.replace(hour=0, minute=0, second=0, microsecond=0) +
-               datetime.timedelta(days=1))
+        cur = start.replace(hour=0, minute=0, second=0, microsecond=0) + datetime.timedelta(days=1)
         while cur.date() < stop.date():
             yield cur, None
             cur += datetime.timedelta(days=1)
@@ -207,8 +199,7 @@ def _get_or_create_group(file, group):
         group = file.get_node(group)
     except tables.NoSuchNodeError:
         parent, newgroup = os.path.split(group)
-        file.create_group(parent, newgroup, 'Data group',
-                          createparents=True)
+        group = file.create_group(parent, newgroup, 'Data group', createparents=True)
     return group
 
 
@@ -218,12 +209,10 @@ def _get_or_create_node(file, group, src_node):
     try:
         node = file.get_node(group, src_node.name)
     except tables.NoSuchNodeError:
-        if type(src_node) == tables.Table:
-            node = file.create_table(group, src_node.name,
-                                     src_node.description, src_node.title)
-        elif type(src_node) == tables.VLArray:
-            node = file.create_vlarray(group, src_node.name, src_node.atom,
-                                       src_node.title)
+        if isinstance(src_node, tables.Table):
+            node = file.create_table(group, src_node.name, src_node.description, src_node.title)
+        elif isinstance(src_node, tables.VLArray):
+            node = file.create_vlarray(group, src_node.name, src_node.atom, src_node.title)
         else:
             raise Exception("Unknown node class: %s" % type(src_node))
 
