@@ -1,12 +1,13 @@
-from mock import sentinel, Mock, patch, MagicMock, call
 import types
 import unittest
 import warnings
 
+from unittest.mock import MagicMock, Mock, call, patch, sentinel
+
 import tables
 
-from sapphire.simulations.base import BaseSimulation
 from sapphire import storage
+from sapphire.simulations.base import BaseSimulation
 
 
 class BaseSimulationTest(unittest.TestCase):
@@ -15,18 +16,19 @@ class BaseSimulationTest(unittest.TestCase):
     def setUp(self, mock_method):
         self.mock_prepare_output_tables = mock_method
         self.cluster = sentinel.cluster
-        self.datafile = sentinel.datafile
+        self.data = sentinel.data
         self.output_path = sentinel.output_path
-        self.N = sentinel.N
+        self.n = sentinel.n
 
-        self.simulation = BaseSimulation(self.cluster, self.datafile,
-                                         self.output_path, self.N)
+        self.simulation = BaseSimulation(self.cluster, self.data,
+                                         self.output_path, self.n,
+                                         progress=False)
 
     def test_init_sets_attributes(self):
         self.assertIs(self.simulation.cluster, self.cluster)
-        self.assertIs(self.simulation.datafile, self.datafile)
+        self.assertIs(self.simulation.data, self.data)
         self.assertIs(self.simulation.output_path, self.output_path)
-        self.assertIs(self.simulation.N, self.N)
+        self.assertIs(self.simulation.n, self.n)
 
     def test_init_calls_prepare_output_tables(self):
         self.mock_prepare_output_tables.assert_called_once_with()
@@ -59,12 +61,8 @@ class BaseSimulationTest(unittest.TestCase):
         mock_store.assert_called_with(1, sentinel.params2,
                                       sentinel.events)
 
-    @patch('sapphire.utils.ProgressBar')
-    def test_generate_shower_parameters(self, mock_progress):
-        # disable progressbar
-        mock_progress.return_value.side_effect = lambda x: x
-
-        self.simulation.N = 10
+    def test_generate_shower_parameters(self):
+        self.simulation.n = 10
         output = self.simulation.generate_shower_parameters()
         self.assertIsInstance(output, types.GeneratorType)
 
@@ -141,8 +139,8 @@ class BaseSimulationTest(unittest.TestCase):
         mock_response.side_effect = [sentinel.observables1,
                                      sentinel.observables2]
 
-        observables = self.simulation.simulate_all_detectors(detectors,
-            sentinel.parameters)
+        observables = self.simulation.simulate_all_detectors(
+            detectors, sentinel.parameters)
 
         expected = [call(sentinel.detector1, sentinel.parameters),
                     call(sentinel.detector2, sentinel.parameters)]
@@ -164,8 +162,7 @@ class BaseSimulationTest(unittest.TestCase):
 
     def test_simulate_gps(self):
         mock_observables = Mock()
-        observables = self.simulation.simulate_gps(mock_observables,
-                                                   Mock(), Mock())
+        self.simulation.simulate_gps(mock_observables, Mock(), Mock())
         self.assertEqual(mock_observables.update.call_count, 1)
         args, kwargs = mock_observables.update.call_args
         gps_dict = args[0]
@@ -203,9 +200,9 @@ class BaseSimulationTest(unittest.TestCase):
         # tests
         station_groups.__getitem__.assert_called_once_with(sentinel.station_id)
 
-        expected = [call('event_id', table.nrows), call('key2', 2.),
-                    call('key1', 1.)]
-        self.assertEqual(table.row.__setitem__.call_args_list, expected)
+        calls = [call('event_id', table.nrows), call('key2', 2.),
+                 call('key1', 1.)]
+        station_groups.asser_has_calls(calls, any_order=True)
         table.row.append.assert_called_once_with()
         table.flush.assert_called_once_with()
         self.assertEqual(idx, table.nrows - 1)
@@ -217,10 +214,11 @@ class BaseSimulationTest(unittest.TestCase):
         observables = {'key1': 1., 'key2': 2.}
         table.colnames = ['key1']
 
-        warnings.simplefilter('error')
-        self.assertRaises(UserWarning,
-                          self.simulation.store_station_observables,
-                          sentinel.station_id, observables)
+        with warnings.catch_warnings(record=True) as warned:
+            warnings.simplefilter('always')
+            self.simulation.store_station_observables(sentinel.station_id,
+                                                      observables)
+        self.assertEqual(len(warned), 1)
 
     @unittest.skip("WIP")
     def test_store_coincidence(self, shower_id, shower_parameters, station_events):
@@ -240,17 +238,22 @@ class BaseSimulationTest(unittest.TestCase):
 
     @unittest.skip("Does not test this unit")
     def test_init_creates_coincidences_output_group(self):
-        self.datafile.create_group.assert_any_call(self.output_path, 'coincidences', createparents=True)
-        self.datafile.create_table.assert_called_with(self.simulation.coincidence_group, 'coincidences', storage.Coincidence)
-        self.assertEqual(self.datafile.create_vlarray.call_count, 2)
-        self.datafile.create_vlarray.assert_any_call(self.simulation.coincidence_group, 'c_index', tables.UInt32Col(shape=2))
+        self.data.create_group.assert_any_call(
+            self.output_path, 'coincidences', createparents=True)
+        self.data.create_table.assert_called_with(
+            self.simulation.coincidence_group, 'coincidences', storage.Coincidence)
+        self.assertEqual(self.data.create_vlarray.call_count, 2)
+        self.data.create_vlarray.assert_any_call(
+            self.simulation.coincidence_group, 'c_index', tables.UInt32Col(shape=2))
 
     @unittest.skip("Does not test this unit")
     def test_init_creates_cluster_output_group(self):
-        self.datafile.create_group.assert_any_call(self.output_path, 'cluster_simulations', createparents=True)
+        self.data.create_group.assert_any_call(
+            self.output_path, 'cluster_simulations', createparents=True)
         # The following tests need a better mock of cluster in order to work.
-        # self.datafile.create_group.assert_any_call(self.simulation.cluster_group, 'station_0')
-        # self.datafile.create_table.assert_any_call(station_group, 'events', storage.ProcessedHisparcEvent, expectedrows=self.N)
+        # self.data.create_group.assert_any_call(self.simulation.cluster_group, 'station_0')
+        # self.data.create_table.assert_any_call(
+        #     station_group, 'events', storage.ProcessedHisparcEvent, expectedrows=self.n)
 
     @unittest.skip("Does not test this unit")
     def test_init_stores_cluster_in_attrs(self):
