@@ -1,38 +1,29 @@
-from mock import sentinel, patch
-import unittest
 import os
+import random
+import unittest
 
-from numpy import array, std, mean
-import tables
+import numpy as np
 
-from sapphire.simulations.detector import (HiSPARCSimulation,
-                                           ErrorlessSimulation)
+from numpy.testing import assert_almost_equal
 
+from sapphire.simulations.detector import ErrorlessSimulation, HiSPARCSimulation
 
 self_path = os.path.dirname(__file__)
 
 
 class HiSPARCSimulationTest(unittest.TestCase):
 
-    @patch.object(HiSPARCSimulation, '_prepare_output_tables')
-    def setUp(self, mock_method):
-        self.mock_prepare_output_tables = mock_method
-        self.cluster = sentinel.cluster
-        self.datafile = sentinel.datafile
-        self.output_path = sentinel.output_path
-        self.N = sentinel.N
-
-        self.simulation = HiSPARCSimulation(cluster=self.cluster,
-                                            datafile=self.datafile,
-                                            output_path=self.output_path,
-                                            N=self.N, seed=1)
+    def setUp(self):
+        self.simulation = HiSPARCSimulation
+        random.seed(1)
+        np.random.seed(1)
 
     def test_simulate_detector_offsets(self):
         self.assertEqual(self.simulation.simulate_detector_offsets(1),
                          [4.49943665734718])
         offsets = self.simulation.simulate_detector_offsets(10000)
-        self.assertAlmostEqual(mean(offsets), 0., 1)
-        self.assertAlmostEqual(std(offsets), 2.77, 2)
+        assert_almost_equal(np.mean(offsets), 0., 1)
+        assert_almost_equal(np.std(offsets), 2.77, 2)
 
     def test_simulate_detector_offset(self):
         self.assertEqual(self.simulation.simulate_detector_offset(),
@@ -48,7 +39,7 @@ class HiSPARCSimulationTest(unittest.TestCase):
 
     def test_simulate_adc_sampling(self):
         self.assertEqual(self.simulation.simulate_adc_sampling(0), 0)
-        self.assertEqual(self.simulation.simulate_adc_sampling(.1), 2.5)
+        self.assertEqual(self.simulation.simulate_adc_sampling(0.1), 2.5)
         self.assertEqual(self.simulation.simulate_adc_sampling(1.25), 2.5)
         self.assertEqual(self.simulation.simulate_adc_sampling(2.5), 2.5)
         self.assertEqual(self.simulation.simulate_adc_sampling(4), 5.)
@@ -67,39 +58,46 @@ class HiSPARCSimulationTest(unittest.TestCase):
                           3.0411502792684506])
 
     def test_simulate_detector_mips(self):
-        corsika_data_path = os.path.join(self_path, 'test_data/corsika.h5')
-        with tables.open_file(corsika_data_path, 'r') as corsika_data:
-            particle = corsika_data.root.groundparticles[0:1]
-            particles = corsika_data.root.groundparticles[:5]
-        self.assertEqual(self.simulation.simulate_detector_mips(particle),
-                         1.0371946215001238)
-        self.assertEqual(self.simulation.simulate_detector_mips(particles),
-                         4.2991522837813028)
+        # Test with single angle
+        assert_almost_equal(self.simulation.simulate_detector_mips(1, 0.5),
+                            1.1818585)
+        assert_almost_equal(self.simulation.simulate_detector_mips(2, 0.2),
+                            1.8313342374)
+
+        # Test with multiple angles
+        assert_almost_equal(self.simulation.simulate_detector_mips(2, np.array([0.5, 1])),
+                            2.58167027)
+        assert_almost_equal(self.simulation.simulate_detector_mips(1, np.array([0.5, 1])),
+                            2.21526297)
+
+        # Test limiting detector length
+        assert_almost_equal(self.simulation.simulate_detector_mips(1, np.radians(90)),
+                            47.6237460)
+        assert_almost_equal(self.simulation.simulate_detector_mips(2, np.array([np.radians(90), np.radians(87)])),
+                            74.6668728)
 
     def test_generate_core_position(self):
         x, y = self.simulation.generate_core_position(500)
-        self.assertAlmostEqual(x, 59.85605947801825)
-        self.assertAlmostEqual(y, 317.2896993591305)
+        assert_almost_equal(x, 59.85605947801825)
+        assert_almost_equal(y, 317.2896993591305)
 
     def test_generate_azimuth(self):
         self.assertEqual(self.simulation.generate_azimuth(),
                          -0.521366120872004)
 
+    def test_generate_energy(self):
+        self.assertEqual(self.simulation.generate_energy(), 136117213526167.64)
+        io = 1e17
+        assert_almost_equal(self.simulation.generate_energy(io, io) / io, 1.)
+        self.assertEqual(self.simulation.generate_energy(alpha=-3), 100005719231473.97)
+
 
 class ErrorlessSimulationTest(HiSPARCSimulationTest):
 
-    @patch.object(HiSPARCSimulation, '_prepare_output_tables')
-    def setUp(self, mock_method):
-        self.mock_prepare_output_tables = mock_method
-        self.cluster = sentinel.cluster
-        self.datafile = sentinel.datafile
-        self.output_path = sentinel.output_path
-        self.N = sentinel.N
-
-        self.simulation = ErrorlessSimulation(cluster=self.cluster,
-                                            datafile=self.datafile,
-                                            output_path=self.output_path,
-                                            N=self.N, seed=1)
+    def setUp(self):
+        self.simulation = ErrorlessSimulation
+        random.seed(1)
+        np.random.seed(1)
 
     def test_simulate_detector_offsets(self):
         self.assertEqual(self.simulation.simulate_detector_offsets(1), [0])
@@ -116,7 +114,7 @@ class ErrorlessSimulationTest(HiSPARCSimulationTest):
 
     def test_simulate_adc_sampling(self):
         self.assertEqual(self.simulation.simulate_adc_sampling(0), 0)
-        self.assertEqual(self.simulation.simulate_adc_sampling(.1), .1)
+        self.assertEqual(self.simulation.simulate_adc_sampling(0.1), 0.1)
         self.assertEqual(self.simulation.simulate_adc_sampling(1.25), 1.25)
         self.assertEqual(self.simulation.simulate_adc_sampling(2.5), 2.5)
         self.assertEqual(self.simulation.simulate_adc_sampling(4), 4.)
@@ -127,13 +125,8 @@ class ErrorlessSimulationTest(HiSPARCSimulationTest):
         self.assertEqual(list(self.simulation.simulate_signal_transport_time(11)), [0] * 11)
 
     def test_simulate_detector_mips(self):
-        corsika_data_path = os.path.join(self_path, 'test_data/corsika.h5')
-        with tables.open_file(corsika_data_path, 'r') as corsika_data:
-            particle = corsika_data.root.groundparticles[0:1]
-            particles = corsika_data.root.groundparticles[:5]
-        self.assertEqual(self.simulation.simulate_detector_mips(particle), 1)
-        self.assertEqual(self.simulation.simulate_detector_mip(particle), 1)
-        self.assertEqual(self.simulation.simulate_detector_mips(particles), 5)
+        self.assertEqual(self.simulation.simulate_detector_mips(1, 0.5), 1)
+        self.assertEqual(self.simulation.simulate_detector_mips(2, 0.2), 2)
 
 
 if __name__ == '__main__':
