@@ -1,7 +1,5 @@
 import unittest
 
-from numpy import pi
-
 from sapphire.transformations import base
 
 
@@ -13,6 +11,8 @@ class DecimalSexagesimalTests(unittest.TestCase):
                              (1, (1, 0, 0)),
                              (30, (30, 0, 0)),
                              (1 / 60., (0, 1, 0)),
+                             (-1 + (30 / 60.), (0, -30, 0)),
+                             (-1 - (30 / 60.) - (30 / 3600.), (-1, -30, -30)),
                              (.5, (0, 30, 0)),
                              (1 / 3600., (0, 0, 1)),
                              (30 / 3600., (0, 0, 30)))
