@@ -4,14 +4,14 @@
     Degrees, radians and hours.
 
 """
-from numpy import radians, degrees
+from numpy import degrees, radians
 
 
 def hours_to_degrees(angle):
     """Converts decimal hours to degrees
 
     :param hours: angle in decimal hours
-    :returns: angle in degrees
+    :return: angle in degrees
 
     """
     return angle * 15.
@@ -21,7 +21,7 @@ def hours_to_radians(angle):
     """Converts decimal hours to radians
 
     :param hours: angle in decimal hours
-    :returns: angle in radians
+    :return: angle in radians
 
     """
     return radians(hours_to_degrees(angle))
@@ -31,7 +31,7 @@ def degrees_to_hours(angle):
     """Converts degrees to decimal hours
 
     :param angle: angle in degrees
-    :returns: angle in decimal hours
+    :return: angle in decimal hours
 
     """
     return angle / 15.
@@ -41,7 +41,7 @@ def radians_to_hours(angle):
     """Converts degrees to decimal hours
 
     :param angle: angle in degrees
-    :returns: angle in decimal hours
+    :return: angle in decimal hours
 
     """
     return degrees_to_hours(degrees(angle))
